import unittest

from fit_tool.utils.crc import crc16


class TestCRC(unittest.TestCase):
    def test_crc16(self):
        data = '123456789'.encode('utf-8')
        result = crc16(data)
        self.assertEqual(result, 0xBB3D)



# nosetests --nocapture  tests/test_demo.py


import unittest

from fit_tool.gen.profile import Profile


class TestFitFile(unittest.TestCase):

    def test_profile(self):
        profile = Profile.get_default_profile();

        for type_name in profile.types_by_name:
            profile_type = profile.types_by_name[type_name]
            print(f'{type_name}', profile_type)



import pytest
from fit_tool.field_component import FieldComponent


class TestFieldComponent:
    @pytest.mark.parametrize("field_id, accumulate, bits, scale, offset", [
        (1, True, 8, 1.0, 0.0),
        (255, False, 16, 100.0, 500.0),
        (0, False, 0, 0.0, 0.0),
        (10, True, 32, 0.001, -500.0),
    ])
    def test_init(self, field_id, accumulate, bits, scale, offset):
        component = FieldComponent(
            field_id=field_id,
            accumulate=accumulate,
            bits=bits,
            scale=scale,
            offset=offset,
        )
        assert component.field_id == field_id
        assert component.accumulate is accumulate
        assert component.bits == bits
        assert component.scale == scale
        assert component.offset == offset



import argparse
import os
from pathlib import Path

import inflection
import jinja2

from fit_tool import SDK_VERSION
from fit_tool.base_type import BaseType
from fit_tool.field import Field
from fit_tool.gen.profile import Profile, Message

DEFAULT_BUILD_PATH = '../../build'


def parse_args():
    """
    Parse command line arguments.
    """

    parser = argparse.ArgumentParser(description="""
This script creates profile related source files.

EXAMPLE:
    gen.py build

""")
    # parser.add_argument("build_dir")

    return parser.parse_args()


def convert_type_name(name: str) -> str:
    name = name.replace('wkt_', 'workout_')

    if name[0].isdigit():
        name = 'n' + name

    name = inflection.camelize(name, uppercase_first_letter=True)
    # name = inflection.underscore(name)

    if name in ['File', 'Switch']:
        name += 'Type'
    return name


def convert_type_name2(name: str) -> str:
    name = name.replace('wkt_', 'workout_')

    if name[0].isdigit():
        name = 'n' + name

    name = inflection.underscore(name)

    if name in ['File', 'Switch']:
        name += 'Type'
    return name.upper()


RESERVED_WORDS = ['new', 'switch', 'enum']


def convert_value_name(name: str) -> str:
    if name[0].isdigit():
        name = 'n' + name
    # name = inflection.camelize(name, uppercase_first_letter=False)
    name = inflection.underscore(name).upper()

    if name in RESERVED_WORDS:
        name = name + '_'
    return name


def un_title(name: str) -> str:
    name = name[0].lower() + name[1:]
    return name


def field_name_to_class_name(message: Message, field: Field, name: str) -> str:
    name = name.replace('wkt_', 'workout_')

    # if not a common field then prefix the message name so we don't get class naming conflicts
    if field.field_id not in [253, 254]:
        name = message.name + '_' + name

    name = inflection.camelize(name, uppercase_first_letter=True)

    if not name.endswith('Field'):
        name += 'Field'
    return name


RESERVED_PROPERTY_NAMES = {'name', 'local_id', 'global_id'}


def field_name_to_property_name(message: Message, name: str) -> str:
    name = name.replace('wkt_', 'workout_')

    if name[0].isdigit():
        name = 'n' + name

    # name = inflection.camelize(name, uppercase_first_letter=False)
    name = inflection.underscore(name)

    if name in RESERVED_PROPERTY_NAMES:
        name = message.name + '_' + name
        name = inflection.underscore(name)

    return name


def subfield_name_to_property_name(field: Field, name: str) -> str:
    name = name.replace('wkt_', 'workout_')

    if field.name == 'target_value':
        if not name.startswith('target_'):
            name = 'target_' + name

    if name[0].isdigit():
        name = 'n' + name

    name = inflection.underscore(name)

    # if name in RESERVED_PROPERTY_NAMES:
    #     name = message.name + '_' + name
    #     name = inflection.camelize(name, uppercase_first_letter=False)

    return name


def get_type_name_from_field(field: Field):
    if field.base_type.name == 'STRING':
        return 'str'
    elif 'FLOAT' in field.base_type.name or (field.scale and field.scale != 1):
        return 'float'
    else:
        return 'int'


def get_field_property_type_name(profile: Profile, field: Field) -> str:
    if field.type_name:
        if 'bool' in field.type_name:
            type_name = 'bool'
        elif 'date_time' == field.type_name:
            type_name = 'int'
        elif 'fit_base_type' in field.type_name:
            type_name = 'int'

        elif field.base_type == BaseType.ENUM:
            type_name = profile.type_class_name_by_name[field.type_name]
        else:
            type_name = get_type_name_from_field(field)
    else:
        type_name = get_type_name_from_field(field)

    # handle arrays
    if field.array_type is None:
        return type_name
    else:
        if field.base_type.name == 'BYTE':
            return 'bytes'
        if type_name == 'str':
            return 'str'
        else:
            return f'list[{type_name}]'


def main():
    build_path = DEFAULT_BUILD_PATH
    profile_path = os.path.join(build_path, 'profile')
    messages_path = os.path.join(profile_path, 'messages')

    messages_dir = Path(messages_path)
    if messages_dir.exists():
        for p in messages_dir.glob('*.py'):
            p.unlink()
        messages_dir.rmdir()

    profile_dir = Path(profile_path)
    if profile_dir.exists():
        for p in profile_dir.glob('*.py'):
            p.unlink()
        profile_dir.rmdir()

    Path(profile_path).mkdir(parents=True, exist_ok=True)
    Path(messages_path).mkdir(parents=True, exist_ok=True)

    profile = Profile.get_default_profile()

    profile.types_by_name.pop('bool')
    profile.types_by_name.pop('fit_base_type')

    profile_types = list({convert_type_name2(k) for k in profile.types_by_name.keys()})
    profile_types.sort()

    profile.type_class_name_by_name = {k: convert_type_name(k) for k in profile.types_by_name.keys()}

    for message in profile.messages_by_name.values():
        message.field_class_name_by_name = {k: field_name_to_class_name(message, message.fields_by_name[k], k) for k in
                                            message.fields_by_name.keys()}
        message.field_property_name_by_name = {k: field_name_to_property_name(message, k) for k in
                                               message.fields_by_name.keys()}
        message.field_property_type_by_name = {k: get_field_property_type_name(profile, v) for k, v in
                                               message.fields_by_name.items()}
        for field in message.fields_by_name.values():
            field.subfield_property_name_by_name = {}
            field.subfield_property_type_by_name = {}
            for sub_field in field.sub_fields:
                ref_field_key = list(sub_field.ref_field_map.keys())[0]
                sub_field.ref_field = message.fields_by_name[ref_field_key]
                sub_field.ref_field_values = sub_field.ref_field_map[ref_field_key]
                field.subfield_property_name_by_name[sub_field.name] = subfield_name_to_property_name(field,
                                                                                                      sub_field.name)
                field.subfield_property_type_by_name[sub_field.name] = get_field_property_type_name(profile, sub_field)

    for k, v in profile.types_by_name.items():
        profile_type = profile.types_by_name[k]
        profile_type.values_by_name = {convert_value_name(k): v for k, v in profile_type.values_by_name.items()}

    env = jinja2.Environment(loader=jinja2.FileSystemLoader('..'), )

    #
    # Gen profile_type.py
    #
    template = env.get_template('gen/templates/template_profile_type.jinja')
    rendering = template.render(profile=profile, profile_types=profile_types,
                                sdk_version=SDK_VERSION)
    filename = os.path.join(profile_path, 'profile_type.py')
    with (open(filename, 'w')) as file_out:
        file_out.write(rendering)

    for name, message in profile.messages_by_name.items():
        template = env.get_template('gen/templates/template_message.jinja')
        rendering = template.render(profile=profile, sdk_version=SDK_VERSION,
                                    class_name=inflection.camelize(message.name + '_message'),
                                    message=message)
        filename = os.path.join(messages_path, f"{inflection.underscore(message.name + '_message')}.py")
        with (open(filename, 'w')) as file_out:
            file_out.write(rendering)

    message_file_names = [inflection.underscore(message_name + '_message') for message_name in
                          profile.messages_by_name.keys()]
    message_class_names = [inflection.camelize(message_name + '_message') for message_name in
                           profile.messages_by_name.keys()]

    template = env.get_template('gen/templates/template_message_factory.jinja')

    message_names = zip(message_class_names, message_file_names)
    rendering = template.render(profile=profile, sdk_version=SDK_VERSION,
                                message_names=message_names, message_class_names=message_class_names)
    filename = os.path.join(messages_path, f"message_factory.py")
    with (open(filename, 'w')) as file_out:
        file_out.write(rendering)

    template = env.get_template('gen/templates/template_common_fields.jinja')
    rendering = template.render(profile=profile, sdk_version=SDK_VERSION)
    filename = os.path.join(messages_path, f"common_fields.py")
    with (open(filename, 'w')) as file_out:
        file_out.write(rendering)


if __name__ == "__main__":
    main()



# Autogenerated. Do not modify.
#
# Profile: 21.60
from typing import List as list
from typing import Optional

from fit_tool.base_type import BaseType
from fit_tool.data_message import DataMessage
from fit_tool.definition_message import DefinitionMessage
from fit_tool.developer_field import DeveloperField
from fit_tool.endian import Endian
from fit_tool.field import Field


class BarometerDataMessage(DataMessage):
    ID = 209
    NAME = 'barometer_data'

    @staticmethod
    def __get_field_size(definition_message: DefinitionMessage, field_id: int) -> int:
        size = 0
        if definition_message:
            field_definition = definition_message.get_field_definition(field_id)
            if field_definition:
                size = field_definition.size

        return size

    def __init__(self, definition_message=None, developer_fields=None, local_id: int = 0,
                 endian: Endian = Endian.LITTLE):
        super().__init__(name=BarometerDataMessage.NAME,
                         global_id=BarometerDataMessage.ID,
                         local_id=definition_message.local_id if definition_message else local_id,
                         endian=definition_message.endian if definition_message else endian,
                         definition_message=definition_message,
                         developer_fields=developer_fields,
                         fields=[
                             TimestampField(
                                 size=self.__get_field_size(definition_message, TimestampField.ID),
                                 growable=definition_message is None),
                             BarometerDataTimestampMsField(
                                 size=self.__get_field_size(definition_message, BarometerDataTimestampMsField.ID),
                                 growable=definition_message is None),
                             BarometerDataSampleTimeOffsetField(
                                 size=self.__get_field_size(definition_message, BarometerDataSampleTimeOffsetField.ID),
                                 growable=definition_message is None),
                             BarometerDataBaroPresField(
                                 size=self.__get_field_size(definition_message, BarometerDataBaroPresField.ID),
                                 growable=definition_message is None)
                         ])

        self.growable = self.definition_message is None

    @classmethod
    def from_bytes(cls, definition_message: DefinitionMessage, developer_fields: list[DeveloperField],
                   bytes_buffer: bytes, offset: int = 0):
        message = cls(definition_message=definition_message, developer_fields=developer_fields)
        message.read_from_bytes(bytes_buffer, offset)
        return message

    # timestamp : milliseconds from January 1st, 1970 at 00:00:00 UTC

    @property
    def timestamp(self) -> Optional[int]:
        field = self.get_field(TimestampField.ID)
        if field and field.is_valid():
            sub_field = field.get_valid_sub_field(self.fields)
            return field.get_value(sub_field=sub_field)
        else:
            return None

    # timestamp : milliseconds from January 1st, 1970 at 00:00:00 UTC

    @timestamp.setter
    def timestamp(self, value: int):
        field = self.get_field(TimestampField.ID)

        if field:
            if value is None:
                field.clear()
            else:
                sub_field = field.get_valid_sub_field(self.fields)
                field.set_value(0, value, sub_field)

    @property
    def timestamp_ms(self) -> Optional[int]:
        field = self.get_field(BarometerDataTimestampMsField.ID)
        if field and field.is_valid():
            sub_field = field.get_valid_sub_field(self.fields)
            return field.get_value(sub_field=sub_field)
        else:
            return None

    @timestamp_ms.setter
    def timestamp_ms(self, value: int):
        field = self.get_field(BarometerDataTimestampMsField.ID)

        if field:
            if value is None:
                field.clear()
            else:
                sub_field = field.get_valid_sub_field(self.fields)
                field.set_value(0, value, sub_field)

    @property
    def sample_time_offset(self) -> Optional[list[int]]:
        field = self.get_field(BarometerDataSampleTimeOffsetField.ID)
        if field and field.is_valid():
            return field.get_values()
        else:
            return None

    @sample_time_offset.setter
    def sample_time_offset(self, value: list[int]):
        field = self.get_field(BarometerDataSampleTimeOffsetField.ID)

        if field:
            if value is None:
                field.clear()
            else:
                field.set_values(value)

    @property
    def baro_pres(self) -> Optional[list[int]]:
        field = self.get_field(BarometerDataBaroPresField.ID)
        if field and field.is_valid():
            return field.get_values()
        else:
            return None

    @baro_pres.setter
    def baro_pres(self, value: list[int]):
        field = self.get_field(BarometerDataBaroPresField.ID)

        if field:
            if value is None:
                field.clear()
            else:
                field.set_values(value)


class TimestampField(Field):
    ID = 253

    def __init__(self, size: int = 0, growable: bool = True):
        super().__init__(
            name='timestamp',
            field_id=self.ID,
            base_type=BaseType.UINT32,
            offset=-631065600000,
            scale=0.001,
            size=size,
            units='ms',
            type_name='date_time',
            growable=growable,
            sub_fields=[
            ]
        )


class BarometerDataTimestampMsField(Field):
    ID = 0

    def __init__(self, size: int = 0, growable: bool = True):
        super().__init__(
            name='timestamp_ms',
            field_id=self.ID,
            base_type=BaseType.UINT16,
            offset=0,
            scale=1,
            size=size,
            units='ms',
            type_name='',
            growable=growable,
            sub_fields=[
            ]
        )


class BarometerDataSampleTimeOffsetField(Field):
    ID = 1

    def __init__(self, size: int = 0, growable: bool = True):
        super().__init__(
            name='sample_time_offset',
            field_id=self.ID,
            base_type=BaseType.UINT16,
            offset=0,
            scale=1,
            size=size,
            units='ms',
            type_name='',
            growable=growable,
            sub_fields=[
            ]
        )


class BarometerDataBaroPresField(Field):
    ID = 2

    def __init__(self, size: int = 0, growable: bool = True):
        super().__init__(
            name='baro_pres',
            field_id=self.ID,
            base_type=BaseType.UINT32,
            offset=0,
            scale=1,
            size=size,
            units='Pa',
            type_name='',
            growable=growable,
            sub_fields=[
            ]
        )



from fit_tool.fit_file import FitFile
from fit_tool.fit_file_builder import FitFileBuilder
from fit_tool.profile.messages.record_message import RecordMessage, RecordHeartRateField, RecordPowerField


def main():
    """The following program reads all the bytes from a FIT formatted file and then
        decodes these bytes to create a FIT file object. We then build a modified FIT file
        based on a variety of criteria (see comments below). Finally we output
        the modified data to a new FIT file.
    """
    path = '../tests/data/sdk/Activity.fit'
    fit_file = FitFile.from_file(path)

    builder = FitFileBuilder(auto_define=False)

    for record in fit_file.records:
        message = record.message
        include_record = True

        if message.global_id == RecordMessage.ID:
            # Remove the heart rate field from all record definition and data messages
            message.remove_field(RecordHeartRateField.ID)

            if isinstance(message, RecordMessage):
                # remove records where the power is too high
                power_field = message.get_field(RecordPowerField.ID)
                if power_field and power_field.is_valid():
                    power = power_field.get_value()
                    if power > 800:
                        include_record = False

        if include_record:
            builder.add(message)

    modified_file = builder.build()
    modified_file.to_file('../tests/out/modified_activity.fit')

    fit_file2 = FitFile.from_file('../tests/out/modified_activity.fit')


if __name__ == "__main__":
    main()



import os.path

from openpyxl import load_workbook

from fit_tool import SDK_VERSION
from fit_tool.base_type import FieldType, BaseType
from fit_tool.field import Field, ArrayType
from fit_tool.utils.logging import logger


class Message:

    def __init__(self, id_, name):
        self.id = id_
        self.name = name
        self.fields_by_id = {}
        self.fields_by_name = {}

    def add_field(self, field: Field):
        self.fields_by_id[field.field_id] = field
        self.fields_by_name[field.name] = field

    def get_field_by_name(self, name):
        return self.fields_by_name.get(name)

    def get_field_by_id(self, id_):
        return self.fields_by_id.get(id_)

    def get_fields(self):
        return list(self.fields_by_id.values())


def parse_array_field(value):
    if value is None:
        return None, None
    if value == '[N]':
        return ArrayType.VARIABLE, None

    return ArrayType.FIXED, int(value[1:-1])


class Profile:

    def __init__(self):
        self.messages_by_id = {}
        self.messages_by_name = {}
        self.types_by_name = {}
        self.type_class_name_by_name = {}

    def get_message_by_name(self, name):
        return self.messages_by_name.get(name)

    def get_message_by_id(self, id_):
        return self.messages_by_id.get(id_)

    def get_type_by_name(self, name):

        # Next check if it is derived type
        type_ = self.types_by_name.get(name)
        if not type_:
            raise Exception('Type: {} not found in profile.'.format(name))
        return type_

    def add_message(self, message):
        self.messages_by_id[message.id] = message
        self.messages_by_name[message.name] = message

    def add_type(self, type_: FieldType):
        self.types_by_name[type_.name] = type_

    # def add_dev_field(self, record):
    #     # array = None
    #     # components = None
    #     # bits = None
    #     # accumulate = None
    #     # fit_base_unit_id = None
    #     # native_mesg_num = None
    #     # native_field_num = None
    #
    #     data = {}
    #     for field in record.fields:
    #         data[field.name] = field.value
    #
    #     developer_data_index = int(data['developer_data_index'])
    #     message = self.get_message_by_id(developer_data_index)
    #
    #     if not message:
    #         message = Message(developer_data_index,
    #                           'dev-' + str(developer_data_index))
    #         self.add_message(message)
    #
    #     # First check if it is a base type
    #     # base_type = BASE_TYPE_BY_ID.get(data.get('fit_base_type_id'))
    #     base_type = None
    #
    #     field = Field(field_id=data.get('field_definition_number'),
    #                   name=data.get('field_name'),
    #                   base_type=base_type,
    #                   scale=data.get('scale', 1),
    #                   offset=data.get('offset', 0),
    #                   units=data.get('units')
    #                   )
    #     message.add_field(field)

    def get_messages(self):
        return list(self.messages_by_id.values())

    @classmethod
    def get_default_profile(cls):

        xlsx_filename = os.path.join(os.path.dirname(__file__), f'Profile_{SDK_VERSION}.xlsx')
        print(f'Loading profile from {xlsx_filename}...')
        profile = Profile.load(xlsx_filename)

        return profile

    @classmethod
    def load(cls, filename):
        profile = cls()
        wb = load_workbook(filename=filename, read_only=True, data_only=True)

        #
        # Parse the Types worksheet
        #
        ws = wb['Types']
        current_type = None
        for index, row in enumerate(ws.rows):
            if index == 0:
                continue

            type_name = row[0].value
            type_base_type_name = row[1].value

            if not type_name and not current_type:
                continue

            if type_name:
                # We are on a new type. Create it and start adding values to
                # it.
                type_base_type = BaseType.from_name(type_base_type_name)
                if not type_base_type:
                    logger.warning(f'Unknown base_type {type_base_type_name}')
                    continue
                current_type = FieldType(type_name, type_base_type)
                profile.add_type(current_type)
            else:
                name = row[2].value

                if name:
                    value = row[3].value
                    if isinstance(value, str):
                        value = int(value, 0)
                    current_type.add_value(name, value)

        profile.add_type(FieldType('bool', BaseType.from_name('uint8')))

        #
        # Parse the Messages worksheet
        #
        ws = wb['Messages']
        current_message = None
        for index, row in enumerate(ws.rows):
            if index == 0:
                continue

            message_name = row[0].value

            if not message_name and not current_message:
                continue

            if message_name:
                # We are on a new message. Create it and start adding values to
                # it.
                message_id = profile.get_type_by_name(
                    'mesg_num').values_by_name[message_name]

                current_message = Message(message_id, message_name)
                profile.add_message(current_message)
            else:
                field_id = row[1].value
                field_name = row[2].value
                field_type_name = row[3].value
                array_type, array_fixed_length = parse_array_field(row[4].value)
                # components = row[5].value
                scale = row[6].value if row[6].value is not None else 1

                # Components are not supported yet
                if isinstance(scale, str):
                    scale = 1

                offset = row[7].value if row[7].value is not None else 0
                units = row[8].value
                if units:
                    units = units.replace('\n', '')

                if units == 'semicircles':
                    units = 'degrees'
                    scale = 2147483648 / 180.0

                if field_type_name == 'date_time':
                    units = 'ms'
                    scale = 1.0 / 1000.0
                    offset = -631065600000

                # bits = row[9].value
                # accumulate = row[10].value
                raw_ref_field_names = row[11].value
                raw_ref_field_values = row[12].value

                ref_field_names = []
                if raw_ref_field_names:
                    ref_field_names = [name.strip()
                                       for name in raw_ref_field_names.split(',')]

                ref_field_values = []
                if raw_ref_field_values:
                    ref_field_values = [value.strip()
                                        for value in raw_ref_field_values.split(',')]

                if field_name == 'custom_target_power_high' or field_name == 'custom_target_power_low':
                    ref_field_names.append('target_type')
                    ref_field_values.append('power_3s')

                    ref_field_names.append('target_type')
                    ref_field_values.append('power_10s')

                    ref_field_names.append('target_type')
                    ref_field_values.append('power_30s')

                    ref_field_names.append('target_type')
                    ref_field_values.append('power_lap')

                if field_name == 'custom_target_heart_rate_high' or field_name == 'custom_target_heart_rate_low':
                    ref_field_names.append('target_type')
                    ref_field_values.append('heart_rate_lap')

                if field_name == 'custom_target_speed_high' or field_name == 'custom_target_speed_low':
                    ref_field_names.append('target_type')
                    ref_field_values.append('speed_lap')

                # comment = row[13].value
                if field_id is not None or field_name is not None:

                    # First check if it is a base type
                    base_type_ = BaseType.from_name(field_type_name)

                    # if not see if it is a derived type in the profile
                    if base_type_:
                        type_ = None
                    else:
                        type_ = profile.get_type_by_name(field_type_name)
                        base_type_ = type_.base_type

                    ref_field_map = {}
                    for ref_index, ref_name in enumerate(ref_field_names):
                        if ref_name in ref_field_map:
                            ref_field_map[ref_name].append(ref_field_values[ref_index])
                        else:
                            ref_field_map[ref_name] = [ref_field_values[ref_index]]

                    field_or_subfield = Field(field_id=field_id,
                                              name=field_name,
                                              base_type=base_type_,
                                              scale=scale,
                                              offset=offset,
                                              units=units,
                                              ref_field_map=ref_field_map,
                                              type_name=field_type_name,
                                              array_type=array_type,
                                              array_fixed_length=array_fixed_length
                                              )

                    # todo: hack for now, should add to class
                    field_or_subfield.type_ = type_

                    is_field = field_id is not None

                    if is_field:
                        field = field_or_subfield
                        current_message.add_field(field)
                    else:
                        subfield = field_or_subfield
                        field.sub_fields.append(subfield)

        cls._resolve_subfield_references(profile)

        wb.close()

        return profile

    @staticmethod
    def _resolve_subfield_references(profile):
        for _, message in profile.messages_by_id.items():
            for _, field in message.fields_by_id.items():
                for subfield in field.sub_fields:
                    if subfield.ref_field_map:
                        resolved_map = {}
                        for key, values in subfield.ref_field_map.items():
                            field = message.get_field_by_name(key)
                            resolved_values = []
                            for item in values:
                                resolved_value = profile.get_type_by_name(field.type_name).get_value_by_name(item)
                                resolved_values.append(resolved_value)
                            resolved_map[key] = resolved_values

                        subfield.ref_field_map = resolved_map

    def create_field(self, msg_name, field_name, value, length=1):
        msg = self.get_message_by_name(msg_name)
        clz = msg.get_field_by_name(field_name)
        clz.length = length
        field = clz(value=value)
        return field

    def create_field_definition(self, msg_name, field_name, length=1):
        msg = self.get_message_by_name(msg_name)
        return msg.get_field_by_name(field_name).to_field_definition(length=length)



# nosetests --nocapture  tests/test_field.py
import os
import unittest

from fit_tool.fit_file import FitFile


class TestActivityFiles(unittest.TestCase):

    def setUp(self):
        super().setUp()
        # stream_handler = logging.StreamHandler(sys.stdout)
        # stream_handler.setFormatter(formatter)
        # logger.addHandler(stream_handler)

    def shortDescription(self):
        return None

    def test_decode_deprecated_activity(self):
        """Test decoding and activity with deprecated messages and fields.
        """
        path = os.path.join(os.path.dirname(__file__), 'data/activity_deprecated_profile.fit')

        with open(path, 'rb') as file_object:
            bytes_buffer = file_object.read()
            fit_file = FitFile.from_bytes(bytes_buffer)
            print(f'Profile version: {fit_file.header.profile_version}')
            fit_file.to_rows()

    def test_decode_multiple_developer_index(self):
        """Test decoding activity with multiple developer indexes
        """
        path = os.path.join(os.path.dirname(__file__), 'data/activity_multiple_developer_index.fit')

        with open(path, 'rb') as file_object:
            bytes_buffer = file_object.read()
            fit_file = FitFile.from_bytes(bytes_buffer)
            print(f'Profile version: {fit_file.header.profile_version}')
            fit_file.to_rows()



import pytest
from fit_tool.utils.conversions import (
    MILLISECONDS_EPOCH_1989_DELTA,
    to_seconds_since_1989_epoch,
    to_milliseconds_since_epoch,
    to_semicircles,
    to_degrees,
)


class TestEpochConversions:
    @pytest.mark.parametrize("ms_epoch, expected_seconds", [
        (MILLISECONDS_EPOCH_1989_DELTA, 0),
        (MILLISECONDS_EPOCH_1989_DELTA + 1000, 1),
        (MILLISECONDS_EPOCH_1989_DELTA + 60000, 60),
    ])
    def test_to_seconds_since_1989_epoch(self, ms_epoch, expected_seconds):
        assert to_seconds_since_1989_epoch(ms_epoch) == expected_seconds

    @pytest.mark.parametrize("seconds, ms_offset", [
        (0, 0),
        (1, 1000),
        (60, 60000),
    ])
    def test_to_milliseconds_since_epoch(self, seconds, ms_offset):
        expected_ms = MILLISECONDS_EPOCH_1989_DELTA + ms_offset
        assert to_milliseconds_since_epoch(seconds) == expected_ms

    @pytest.mark.parametrize("original_ms", [
        1700000000000,
        MILLISECONDS_EPOCH_1989_DELTA,
        MILLISECONDS_EPOCH_1989_DELTA + 1234,
    ])
    def test_round_trip_conversion(self, original_ms):
        seconds = to_seconds_since_1989_epoch(original_ms)
        result_ms = to_milliseconds_since_epoch(seconds)
        assert result_ms == (original_ms // 1000) * 1000


class TestCoordinateConversions:
    @pytest.mark.parametrize("degrees, expected_semicircles", [
        (0, 0),
        (180, 2147483648),
        (-180, -2147483648),
        (90, 1073741824),
    ])
    def test_to_semicircles(self, degrees, expected_semicircles):
        assert to_semicircles(degrees) == expected_semicircles

    @pytest.mark.parametrize("semicircles, expected_degrees", [
        (0, 0.0),
        (2147483648, 180.0),
        (-2147483648, -180.0),
        (1073741824, 90.0),
    ])
    def test_to_degrees(self, semicircles, expected_degrees):
        assert to_degrees(semicircles) == pytest.approx(expected_degrees)

    @pytest.mark.parametrize("degrees", [0, 45, 90, -90, 180, -180, 37.7749, -122.4194])
    def test_round_trip_conversion(self, degrees):
        semicircles = to_semicircles(degrees)
        result = to_degrees(semicircles)
        assert result == pytest.approx(degrees, abs=1e-4)



# nosetests --nocapture  tests/test_fit_file_header.py

import unittest

from fit_tool.fit_file_header import ProtocolVersion, ProfileVersion, FitFileHeader


class TestFitFileHeader(unittest.TestCase):

    def shortDescription(self):
        return None

    def test_protocol_version(self):
        protocol_version = ProtocolVersion(2, 3)
        self.assertEqual('2.3', str(protocol_version))

        bytes1 = protocol_version.to_bytes()

        pv2 = ProtocolVersion.from_bytes(bytes1)
        bytes2 = pv2.to_bytes()

        self.assertEqual(bytes1, bytes2)

    def test_profile_version(self):
        profile_version = ProfileVersion(21, 60)
        self.assertEqual('21.60', str(profile_version))

        bytes1 = profile_version.to_bytes()

        pv2 = ProfileVersion.from_bytes(bytes1)
        bytes2 = pv2.to_bytes()

        self.assertEqual(bytes1, bytes2)

    def test_generate_crc(self):
        protocol_version = ProtocolVersion(2, 3)
        profile_version = ProfileVersion(21, 60)

        FitFileHeader.generate_crc(protocol_version, profile_version, 6552343)

    def test_conversion_without_crc(self):
        header1 = FitFileHeader(protocol_version=ProtocolVersion(2, 3), profile_version=ProfileVersion(21, 60),
                                records_size=4294967295)
        bytes1 = header1.to_bytes()
        self.assertEqual(12, len(bytes1))

        header2 = FitFileHeader.from_bytes(bytes1)
        bytes2 = header2.to_bytes()

        self.assertEqual(bytes1, bytes2)

    def test_conversion_with_crc(self):
        header1 = FitFileHeader(protocol_version=ProtocolVersion(2, 3), profile_version=ProfileVersion(21, 60),
                                records_size=4294967295, crc=500)
        bytes1 = header1.to_bytes()
        self.assertEqual(14, len(bytes1))

        header2 = FitFileHeader.from_bytes(bytes1)
        bytes2 = header2.to_bytes()

        self.assertEqual(bytes1, bytes2)



# nosetests --nocapture  tests/test_field.py

import unittest

from fit_tool.definition_message import DefinitionMessage
from fit_tool.profile.messages.workout_step_message import WorkoutStepMessage
from fit_tool.profile.profile_type import WorkoutStepDuration


class TestDataMessage(unittest.TestCase):

    def shortDescription(self):
        return None

    def test_data_message_conversions(self):
        dm1 = WorkoutStepMessage()
        dm1.workout_step_name = 'test'
        self.assertEqual('test', dm1.workout_step_name)

        bytes1 = dm1.to_bytes()

        definition_message = DefinitionMessage.from_data_message(dm1)
        dm2 = WorkoutStepMessage(definition_message=definition_message)
        dm2.read_from_bytes(bytes1)
        bytes2 = dm2.to_bytes()

        self.assertEqual('test', dm2.workout_step_name)

        self.assertEqual(bytes2, bytes1)

    def test_to_row(self):
        dm1 = WorkoutStepMessage()
        dm1.workout_step_name = 'test'
        dm1.duration_type = WorkoutStepDuration.DISTANCE

        row = dm1.to_row()
        print(row)



# Autogenerated. Do not modify.
#
# Profile: 21.60
from typing import List as list
from typing import Optional

from fit_tool.base_type import BaseType
from fit_tool.data_message import DataMessage
from fit_tool.definition_message import DefinitionMessage
from fit_tool.developer_field import DeveloperField
from fit_tool.endian import Endian
from fit_tool.field import Field


class MemoGlobMessage(DataMessage):
    ID = 145
    NAME = 'memo_glob'

    @staticmethod
    def __get_field_size(definition_message: DefinitionMessage, field_id: int) -> int:
        size = 0
        if definition_message:
            field_definition = definition_message.get_field_definition(field_id)
            if field_definition:
                size = field_definition.size

        return size

    def __init__(self, definition_message=None, developer_fields=None, local_id: int = 0,
                 endian: Endian = Endian.LITTLE):
        super().__init__(name=MemoGlobMessage.NAME,
                         global_id=MemoGlobMessage.ID,
                         local_id=definition_message.local_id if definition_message else local_id,
                         endian=definition_message.endian if definition_message else endian,
                         definition_message=definition_message,
                         developer_fields=developer_fields,
                         fields=[
                             MemoGlobPartIndexField(
                                 size=self.__get_field_size(definition_message, MemoGlobPartIndexField.ID),
                                 growable=definition_message is None),
                             MemoGlobMemoField(
                                 size=self.__get_field_size(definition_message, MemoGlobMemoField.ID),
                                 growable=definition_message is None),
                             MemoGlobMessageNumberField(
                                 size=self.__get_field_size(definition_message, MemoGlobMessageNumberField.ID),
                                 growable=definition_message is None),
                             MemoGlobMessageIndexField(
                                 size=self.__get_field_size(definition_message, MemoGlobMessageIndexField.ID),
                                 growable=definition_message is None)
                         ])

        self.growable = self.definition_message is None

    @classmethod
    def from_bytes(cls, definition_message: DefinitionMessage, developer_fields: list[DeveloperField],
                   bytes_buffer: bytes, offset: int = 0):
        message = cls(definition_message=definition_message, developer_fields=developer_fields)
        message.read_from_bytes(bytes_buffer, offset)
        return message

    @property
    def part_index(self) -> Optional[int]:
        field = self.get_field(MemoGlobPartIndexField.ID)
        if field and field.is_valid():
            sub_field = field.get_valid_sub_field(self.fields)
            return field.get_value(sub_field=sub_field)
        else:
            return None

    @part_index.setter
    def part_index(self, value: int):
        field = self.get_field(MemoGlobPartIndexField.ID)

        if field:
            if value is None:
                field.clear()
            else:
                sub_field = field.get_valid_sub_field(self.fields)
                field.set_value(0, value, sub_field)

    @property
    def memo(self) -> Optional[bytes]:
        field = self.get_field(MemoGlobMemoField.ID)
        if field and field.is_valid():
            return field.get_values()
        else:
            return None

    @memo.setter
    def memo(self, value: bytes):
        field = self.get_field(MemoGlobMemoField.ID)

        if field:
            if value is None:
                field.clear()
            else:
                field.set_values(value)

    @property
    def message_number(self) -> Optional[int]:
        field = self.get_field(MemoGlobMessageNumberField.ID)
        if field and field.is_valid():
            sub_field = field.get_valid_sub_field(self.fields)
            return field.get_value(sub_field=sub_field)
        else:
            return None

    @message_number.setter
    def message_number(self, value: int):
        field = self.get_field(MemoGlobMessageNumberField.ID)

        if field:
            if value is None:
                field.clear()
            else:
                sub_field = field.get_valid_sub_field(self.fields)
                field.set_value(0, value, sub_field)

    @property
    def message_index(self) -> Optional[int]:
        field = self.get_field(MemoGlobMessageIndexField.ID)
        if field and field.is_valid():
            sub_field = field.get_valid_sub_field(self.fields)
            return field.get_value(sub_field=sub_field)
        else:
            return None

    @message_index.setter
    def message_index(self, value: int):
        field = self.get_field(MemoGlobMessageIndexField.ID)

        if field:
            if value is None:
                field.clear()
            else:
                sub_field = field.get_valid_sub_field(self.fields)
                field.set_value(0, value, sub_field)


class MemoGlobPartIndexField(Field):
    ID = 250

    def __init__(self, size: int = 0, growable: bool = True):
        super().__init__(
            name='part_index',
            field_id=self.ID,
            base_type=BaseType.UINT32,
            offset=0,
            scale=1,
            size=size,
            growable=growable,
            sub_fields=[
            ]
        )


class MemoGlobMemoField(Field):
    ID = 0

    def __init__(self, size: int = 0, growable: bool = True):
        super().__init__(
            name='memo',
            field_id=self.ID,
            base_type=BaseType.BYTE,
            offset=0,
            scale=1,
            size=size,
            growable=growable,
            sub_fields=[
            ]
        )


class MemoGlobMessageNumberField(Field):
    ID = 1

    def __init__(self, size: int = 0, growable: bool = True):
        super().__init__(
            name='message_number',
            field_id=self.ID,
            base_type=BaseType.UINT16,
            offset=0,
            scale=1,
            size=size,
            growable=growable,
            sub_fields=[
            ]
        )


class MemoGlobMessageIndexField(Field):
    ID = 2

    def __init__(self, size: int = 0, growable: bool = True):
        super().__init__(
            name='message_index',
            field_id=self.ID,
            base_type=BaseType.UINT16,
            offset=0,
            scale=1,
            size=size,
            growable=growable,
            sub_fields=[
            ]
        )



# Autogenerated. Do not modify.
#
# Profile: 21.60
from typing import List as list
from typing import Optional

from fit_tool.base_type import BaseType
from fit_tool.data_message import DataMessage
from fit_tool.definition_message import DefinitionMessage
from fit_tool.developer_field import DeveloperField
from fit_tool.endian import Endian
from fit_tool.field import Field
from fit_tool.profile.profile_type import *


class CoursePointMessage(DataMessage):
    ID = 32
    NAME = 'course_point'

    @staticmethod
    def __get_field_size(definition_message: DefinitionMessage, field_id: int) -> int:
        size = 0
        if definition_message:
            field_definition = definition_message.get_field_definition(field_id)
            if field_definition:
                size = field_definition.size

        return size

    def __init__(self, definition_message=None, developer_fields=None, local_id: int = 0,
                 endian: Endian = Endian.LITTLE):
        super().__init__(name=CoursePointMessage.NAME,
                         global_id=CoursePointMessage.ID,
                         local_id=definition_message.local_id if definition_message else local_id,
                         endian=definition_message.endian if definition_message else endian,
                         definition_message=definition_message,
                         developer_fields=developer_fields,
                         fields=[
                             MessageIndexField(
                                 size=self.__get_field_size(definition_message, MessageIndexField.ID),
                                 growable=definition_message is None),
                             CoursePointTimestampField(
                                 size=self.__get_field_size(definition_message, CoursePointTimestampField.ID),
                                 growable=definition_message is None),
                             CoursePointPositionLatField(
                                 size=self.__get_field_size(definition_message, CoursePointPositionLatField.ID),
                                 growable=definition_message is None),
                             CoursePointPositionLongField(
                                 size=self.__get_field_size(definition_message, CoursePointPositionLongField.ID),
                                 growable=definition_message is None),
                             CoursePointDistanceField(
                                 size=self.__get_field_size(definition_message, CoursePointDistanceField.ID),
                                 growable=definition_message is None),
                             CoursePointTypeField(
                                 size=self.__get_field_size(definition_message, CoursePointTypeField.ID),
                                 growable=definition_message is None),
                             CoursePointNameField(
                                 size=self.__get_field_size(definition_message, CoursePointNameField.ID),
                                 growable=definition_message is None),
                             CoursePointFavoriteField(
                                 size=self.__get_field_size(definition_message, CoursePointFavoriteField.ID),
                                 growable=definition_message is None)
                         ])

        self.growable = self.definition_message is None

    @classmethod
    def from_bytes(cls, definition_message: DefinitionMessage, developer_fields: list[DeveloperField],
                   bytes_buffer: bytes, offset: int = 0):
        message = cls(definition_message=definition_message, developer_fields=developer_fields)
        message.read_from_bytes(bytes_buffer, offset)
        return message

    @property
    def message_index(self) -> Optional[int]:
        field = self.get_field(MessageIndexField.ID)
        if field and field.is_valid():
            sub_field = field.get_valid_sub_field(self.fields)
            return field.get_value(sub_field=sub_field)
        else:
            return None

    @message_index.setter
    def message_index(self, value: int):
        field = self.get_field(MessageIndexField.ID)

        if field:
            if value is None:
                field.clear()
            else:
                sub_field = field.get_valid_sub_field(self.fields)
                field.set_value(0, value, sub_field)

    # timestamp : milliseconds from January 1st, 1970 at 00:00:00 UTC

    @property
    def timestamp(self) -> Optional[int]:
        field = self.get_field(CoursePointTimestampField.ID)
        if field and field.is_valid():
            sub_field = field.get_valid_sub_field(self.fields)
            return field.get_value(sub_field=sub_field)
        else:
            return None

    # timestamp : milliseconds from January 1st, 1970 at 00:00:00 UTC

    @timestamp.setter
    def timestamp(self, value: int):
        field = self.get_field(CoursePointTimestampField.ID)

        if field:
            if value is None:
                field.clear()
            else:
                sub_field = field.get_valid_sub_field(self.fields)
                field.set_value(0, value, sub_field)

    @property
    def position_lat(self) -> Optional[float]:
        field = self.get_field(CoursePointPositionLatField.ID)
        if field and field.is_valid():
            sub_field = field.get_valid_sub_field(self.fields)
            return field.get_value(sub_field=sub_field)
        else:
            return None

    @position_lat.setter
    def position_lat(self, value: float):
        field = self.get_field(CoursePointPositionLatField.ID)

        if field:
            if value is None:
                field.clear()
            else:
                sub_field = field.get_valid_sub_field(self.fields)
                field.set_value(0, value, sub_field)

    @property
    def position_long(self) -> Optional[float]:
        field = self.get_field(CoursePointPositionLongField.ID)
        if field and field.is_valid():
            sub_field = field.get_valid_sub_field(self.fields)
            return field.get_value(sub_field=sub_field)
        else:
            return None

    @position_long.setter
    def position_long(self, value: float):
        field = self.get_field(CoursePointPositionLongField.ID)

        if field:
            if value is None:
                field.clear()
            else:
                sub_field = field.get_valid_sub_field(self.fields)
                field.set_value(0, value, sub_field)

    @property
    def distance(self) -> Optional[float]:
        field = self.get_field(CoursePointDistanceField.ID)
        if field and field.is_valid():
            sub_field = field.get_valid_sub_field(self.fields)
            return field.get_value(sub_field=sub_field)
        else:
            return None

    @distance.setter
    def distance(self, value: float):
        field = self.get_field(CoursePointDistanceField.ID)

        if field:
            if value is None:
                field.clear()
            else:
                sub_field = field.get_valid_sub_field(self.fields)
                field.set_value(0, value, sub_field)

    @property
    def type(self) -> Optional[CoursePoint]:
        field = self.get_field(CoursePointTypeField.ID)
        if field and field.is_valid():
            sub_field = field.get_valid_sub_field(self.fields)
            return field.get_value(sub_field=sub_field)
        else:
            return None

    @type.setter
    def type(self, value: CoursePoint):
        field = self.get_field(CoursePointTypeField.ID)

        if field:
            if value is None:
                field.clear()
            else:
                sub_field = field.get_valid_sub_field(self.fields)
                field.set_value(0, value, sub_field)

    @property
    def course_point_name(self) -> Optional[str]:
        field = self.get_field(CoursePointNameField.ID)
        if field and field.is_valid():
            sub_field = field.get_valid_sub_field(self.fields)
            return field.get_value(sub_field=sub_field)
        else:
            return None

    @course_point_name.setter
    def course_point_name(self, value: str):
        field = self.get_field(CoursePointNameField.ID)

        if field:
            if value is None:
                field.clear()
            else:
                sub_field = field.get_valid_sub_field(self.fields)
                field.set_value(0, value, sub_field)

    @property
    def favorite(self) -> Optional[bool]:
        field = self.get_field(CoursePointFavoriteField.ID)
        if field and field.is_valid():
            sub_field = field.get_valid_sub_field(self.fields)
            return field.get_value(sub_field=sub_field)
        else:
            return None

    @favorite.setter
    def favorite(self, value: bool):
        field = self.get_field(CoursePointFavoriteField.ID)

        if field:
            if value is None:
                field.clear()
            else:
                sub_field = field.get_valid_sub_field(self.fields)
                field.set_value(0, value, sub_field)


class MessageIndexField(Field):
    ID = 254

    def __init__(self, size: int = 0, growable: bool = True):
        super().__init__(
            name='message_index',
            field_id=self.ID,
            base_type=BaseType.UINT16,
            offset=0,
            scale=1,
            size=size,
            growable=growable,
            sub_fields=[
            ]
        )


class CoursePointTimestampField(Field):
    ID = 1

    def __init__(self, size: int = 0, growable: bool = True):
        super().__init__(
            name='timestamp',
            field_id=self.ID,
            base_type=BaseType.UINT32,
            offset=-631065600000,
            scale=0.001,
            size=size,
            units='ms',
            type_name='date_time',
            growable=growable,
            sub_fields=[
            ]
        )


class CoursePointPositionLatField(Field):
    ID = 2

    def __init__(self, size: int = 0, growable: bool = True):
        super().__init__(
            name='position_lat',
            field_id=self.ID,
            base_type=BaseType.SINT32,
            offset=0,
            scale=11930464.711111112,
            size=size,
            units='degrees',
            type_name='',
            growable=growable,
            sub_fields=[
            ]
        )


class CoursePointPositionLongField(Field):
    ID = 3

    def __init__(self, size: int = 0, growable: bool = True):
        super().__init__(
            name='position_long',
            field_id=self.ID,
            base_type=BaseType.SINT32,
            offset=0,
            scale=11930464.711111112,
            size=size,
            units='degrees',
            type_name='',
            growable=growable,
            sub_fields=[
            ]
        )


class CoursePointDistanceField(Field):
    ID = 4

    def __init__(self, size: int = 0, growable: bool = True):
        super().__init__(
            name='distance',
            field_id=self.ID,
            base_type=BaseType.UINT32,
            offset=0,
            scale=100,
            size=size,
            units='m',
            type_name='',
            growable=growable,
            sub_fields=[
            ]
        )


class CoursePointTypeField(Field):
    ID = 5

    def __init__(self, size: int = 0, growable: bool = True):
        super().__init__(
            name='type',
            field_id=self.ID,
            base_type=BaseType.ENUM,
            offset=0,
            scale=1,
            size=size,
            growable=growable,
            sub_fields=[
            ]
        )


class CoursePointNameField(Field):
    ID = 6

    def __init__(self, size: int = 0, growable: bool = True):
        super().__init__(
            name='name',
            field_id=self.ID,
            base_type=BaseType.STRING,
            offset=0,
            scale=1,
            size=size,
            growable=growable,
            sub_fields=[
            ]
        )


class CoursePointFavoriteField(Field):
    ID = 8

    def __init__(self, size: int = 0, growable: bool = True):
        super().__init__(
            name='favorite',
            field_id=self.ID,
            base_type=BaseType.UINT8,
            offset=0,
            scale=1,
            size=size,
            growable=growable,
            sub_fields=[
            ]
        )



import pytest
from fit_tool.base_type import BaseType, FieldType


class TestBaseTypeSize:
    @pytest.mark.parametrize("base_type, expected_size", [
        (BaseType.ENUM, 1),
        (BaseType.SINT8, 1),
        (BaseType.UINT8, 1),
        (BaseType.STRING, 1),
        (BaseType.UINT8Z, 1),
        (BaseType.BYTE, 1),
        (BaseType.SINT16, 2),
        (BaseType.UINT16, 2),
        (BaseType.UINT16Z, 2),
        (BaseType.SINT32, 4),
        (BaseType.UINT32, 4),
        (BaseType.FLOAT32, 4),
        (BaseType.UINT32Z, 4),
        (BaseType.FLOAT64, 8),
        (BaseType.SINT64, 8),
        (BaseType.UINT64, 8),
        (BaseType.UINT64Z, 8),
    ])
    def test_size(self, base_type, expected_size):
        assert base_type.size == expected_size


class TestBaseTypeIsInteger:
    def test_is_integer_true(self):
        integer_types = [
            BaseType.SINT8, BaseType.UINT8, BaseType.SINT16, BaseType.UINT16,
            BaseType.UINT16Z, BaseType.SINT32, BaseType.UINT32, BaseType.UINT32Z,
            BaseType.SINT64, BaseType.UINT64, BaseType.UINT64Z,
        ]
        for t in integer_types:
            assert t.is_integer() is True

    def test_is_integer_false(self):
        non_integer_types = [
            BaseType.ENUM, BaseType.STRING, BaseType.FLOAT32, BaseType.FLOAT64, BaseType.BYTE,
        ]
        for t in non_integer_types:
            assert t.is_integer() is False


class TestBaseTypeIsSignedInteger:
    def test_is_signed_integer_true(self):
        signed_types = [BaseType.SINT8, BaseType.SINT16, BaseType.SINT32, BaseType.SINT64]
        for t in signed_types:
            assert t.is_signed_integer() is True

    def test_is_signed_integer_false(self):
        unsigned_types = [
            BaseType.UINT8, BaseType.UINT16, BaseType.UINT32, BaseType.UINT64,
            BaseType.UINT8Z, BaseType.UINT16Z, BaseType.UINT32Z, BaseType.UINT64Z,
            BaseType.ENUM, BaseType.STRING, BaseType.FLOAT32, BaseType.FLOAT64, BaseType.BYTE,
        ]
        for t in unsigned_types:
            assert t.is_signed_integer() is False


class TestBaseTypeIsBig:
    def test_is_big_true(self):
        big_types = [BaseType.SINT64, BaseType.UINT64, BaseType.UINT64Z]
        for t in big_types:
            assert t.is_big() is True

    def test_is_big_false(self):
        small_types = [
            BaseType.ENUM, BaseType.SINT8, BaseType.UINT8, BaseType.SINT16,
            BaseType.UINT16, BaseType.SINT32, BaseType.UINT32, BaseType.STRING,
            BaseType.FLOAT32, BaseType.FLOAT64, BaseType.UINT8Z, BaseType.UINT16Z,
            BaseType.UINT32Z, BaseType.BYTE,
        ]
        for t in small_types:
            assert t.is_big() is False


class TestBaseTypeIsString:
    def test_is_string_true(self):
        assert BaseType.STRING.is_string() is True

    def test_is_string_false(self):
        for t in BaseType:
            if t != BaseType.STRING:
                assert t.is_string() is False


class TestBaseTypeIsFloat:
    def test_is_float_true(self):
        assert BaseType.FLOAT32.is_float() is True
        assert BaseType.FLOAT64.is_float() is True

    def test_is_float_false(self):
        non_float_types = [
            BaseType.ENUM, BaseType.SINT8, BaseType.UINT8, BaseType.SINT16,
            BaseType.UINT16, BaseType.SINT32, BaseType.UINT32, BaseType.STRING,
            BaseType.UINT8Z, BaseType.UINT16Z, BaseType.UINT32Z, BaseType.BYTE,
            BaseType.SINT64, BaseType.UINT64, BaseType.UINT64Z,
        ]
        for t in non_float_types:
            assert t.is_float() is False


class TestBaseTypeIsValid:
    def test_is_valid_none(self):
        assert BaseType.UINT8.is_valid(None) is False

    def test_is_valid_string_type(self):
        assert BaseType.STRING.is_valid("test") is True

    def test_is_valid_float_type(self):
        assert BaseType.FLOAT32.is_valid(3.14) is True
        assert BaseType.FLOAT64.is_valid(3.14159265359) is True

    def test_is_valid_in_range(self):
        assert BaseType.UINT8.is_valid(0) is True
        assert BaseType.UINT8.is_valid(255) is True
        assert BaseType.SINT8.is_valid(-128) is True
        assert BaseType.SINT8.is_valid(127) is True

    def test_is_valid_out_of_range(self):
        assert BaseType.UINT8.is_valid(-1) is False
        assert BaseType.UINT8.is_valid(256) is False
        assert BaseType.SINT8.is_valid(-129) is False
        assert BaseType.SINT8.is_valid(128) is False


class TestBaseTypeInvalidRawValue:
    def test_invalid_raw_value_1_byte(self):
        assert BaseType.ENUM.invalid_raw_value() == 0xff
        assert BaseType.SINT8.invalid_raw_value() == 0x7f
        assert BaseType.UINT8.invalid_raw_value() == 0xff
        assert BaseType.UINT8Z.invalid_raw_value() == 0x00
        assert BaseType.BYTE.invalid_raw_value() == 0xff

    def test_invalid_raw_value_2_byte(self):
        assert BaseType.SINT16.invalid_raw_value() == 0x7fff
        assert BaseType.UINT16.invalid_raw_value() == 0xffff
        assert BaseType.UINT16Z.invalid_raw_value() == 0x0000

    def test_invalid_raw_value_4_byte(self):
        assert BaseType.SINT32.invalid_raw_value() == 0x7fffffff
        assert BaseType.UINT32.invalid_raw_value() == 0xffffffff
        assert BaseType.UINT32Z.invalid_raw_value() == 0x00000000
        assert BaseType.FLOAT32.invalid_raw_value() == 0xffffffff

    def test_invalid_raw_value_8_byte(self):
        assert BaseType.SINT64.invalid_raw_value() == 0x7fffffffffffffff
        assert BaseType.UINT64.invalid_raw_value() == 0xffffffffffffffff
        assert BaseType.UINT64Z.invalid_raw_value() == 0x0000000000000000
        assert BaseType.FLOAT64.invalid_raw_value() == 0xffffffffffffffff

    def test_invalid_raw_value_string(self):
        assert BaseType.STRING.invalid_raw_value() == 0x00


class TestBaseTypeMinMax:
    @pytest.mark.parametrize("base_type, expected_max", [
        (BaseType.ENUM, 0xff),
        (BaseType.SINT8, 0x7f),
        (BaseType.UINT8, 0xff),
        (BaseType.SINT16, 0x7fff),
        (BaseType.UINT16, 0xffff),
        (BaseType.SINT32, 0x7fffffff),
        (BaseType.UINT32, 0xffffffff),
        (BaseType.STRING, None),
        (BaseType.FLOAT32, None),
        (BaseType.FLOAT64, None),
        (BaseType.UINT8Z, 0xff),
        (BaseType.UINT16Z, 0xffff),
        (BaseType.UINT32Z, 0xffffffff),
        (BaseType.BYTE, 0xff),
        (BaseType.SINT64, 0x7fffffffffffffff),
        (BaseType.UINT64, 0xffffffffffffffff),
        (BaseType.UINT64Z, 0xffffffffffffffff),
    ])
    def test_max_values(self, base_type, expected_max):
        assert base_type.max == expected_max

    @pytest.mark.parametrize("base_type, expected_min", [
        (BaseType.ENUM, 0x00),
        (BaseType.SINT8, -0x80),
        (BaseType.UINT8, 0x00),
        (BaseType.SINT16, -0x8000),
        (BaseType.UINT16, 0x0000),
        (BaseType.SINT32, -0x80000000),
        (BaseType.UINT32, 0x00000000),
        (BaseType.STRING, None),
        (BaseType.FLOAT32, None),
        (BaseType.FLOAT64, None),
        (BaseType.UINT8Z, 0x00),
        (BaseType.UINT16Z, 0x0000),
        (BaseType.UINT32Z, 0x00000000),
        (BaseType.BYTE, 0x00),
        (BaseType.SINT64, -0x8000000000000000),
        (BaseType.UINT64, 0x0000000000000000),
        (BaseType.UINT64Z, 0x0000000000000000),
    ])
    def test_min_values(self, base_type, expected_min):
        assert base_type.min == expected_min


class TestBaseTypeFromName:
    def test_from_name_valid(self):
        assert BaseType.from_name('enum') == BaseType.ENUM
        assert BaseType.from_name('sint8') == BaseType.SINT8
        assert BaseType.from_name('uint8') == BaseType.UINT8
        assert BaseType.from_name('sint16') == BaseType.SINT16
        assert BaseType.from_name('uint16') == BaseType.UINT16
        assert BaseType.from_name('sint32') == BaseType.SINT32
        assert BaseType.from_name('uint32') == BaseType.UINT32
        assert BaseType.from_name('string') == BaseType.STRING
        assert BaseType.from_name('float32') == BaseType.FLOAT32
        assert BaseType.from_name('float64') == BaseType.FLOAT64
        assert BaseType.from_name('uint8z') == BaseType.UINT8Z
        assert BaseType.from_name('uint16z') == BaseType.UINT16Z
        assert BaseType.from_name('uint32z') == BaseType.UINT32Z
        assert BaseType.from_name('byte') == BaseType.BYTE
        assert BaseType.from_name('sint64') == BaseType.SINT64
        assert BaseType.from_name('uint64') == BaseType.UINT64
        assert BaseType.from_name('uint64z') == BaseType.UINT64Z

    def test_from_name_invalid(self):
        assert BaseType.from_name('invalid') is None
        assert BaseType.from_name('') is None
        assert BaseType.from_name('UINT8') is None


class TestFieldType:
    def test_init(self):
        ft = FieldType('test_type', BaseType.UINT8)
        assert ft.name == 'test_type'
        assert ft.base_type == BaseType.UINT8
        assert ft.names_by_value == {}
        assert ft.values_by_name == {}

    def test_add_value(self):
        ft = FieldType('activity_type', BaseType.ENUM)
        ft.add_value('running', 0)
        ft.add_value('cycling', 1)
        ft.add_value('swimming', 2)

        assert ft.names_by_value[0] == 'running'
        assert ft.names_by_value[1] == 'cycling'
        assert ft.names_by_value[2] == 'swimming'
        assert ft.values_by_name['running'] == 0
        assert ft.values_by_name['cycling'] == 1
        assert ft.values_by_name['swimming'] == 2

    def test_get_value_by_name(self):
        ft = FieldType('sport', BaseType.ENUM)
        ft.add_value('running', 0)
        ft.add_value('cycling', 1)

        assert ft.get_value_by_name('running') == 0
        assert ft.get_value_by_name('cycling') == 1
        assert ft.get_value_by_name('unknown') is None

    def test_get_name_by_value(self):
        ft = FieldType('sport', BaseType.ENUM)
        ft.add_value('running', 0)
        ft.add_value('cycling', 1)

        assert ft.get_name_by_value(0) == 'running'
        assert ft.get_name_by_value(1) == 'cycling'
        assert ft.get_name_by_value(99) is None



from fit_tool.fit_file import FitFile


def main():
    """ The following code reads all the bytes from a FIT formatted file and then decodes these bytes to
        create a FIT file object. We then convert the FIT data to a human-readable CSV file.
    """
    path = '../tests/data/sdk/Activity.fit'
    fit_file = FitFile.from_file(path)

    out_path = '../tests/data/sdk/Activity.csv'
    fit_file.to_csv(out_path)


if __name__ == "__main__":
    main()



import datetime
import os
import struct
import unittest

TEST_DIR = os.path.dirname(__file__)
OUT_DIR = os.path.join(TEST_DIR, 'out')
os.makedirs(OUT_DIR, exist_ok=True)

from fit_tool.base_type import BaseType
from fit_tool.definition_message import DefinitionMessage
from fit_tool.developer_field import DeveloperField
from fit_tool.fit_file import FitFile
from fit_tool.fit_file_builder import FitFileBuilder
from fit_tool.fit_file_header import FitFileHeader
from fit_tool.message import Message
from fit_tool.profile.messages.developer_data_id_message import DeveloperDataIdMessage
from fit_tool.profile.messages.event_message import EventMessage
from fit_tool.profile.messages.field_description_message import FieldDescriptionMessage
from fit_tool.profile.messages.file_id_message import FileIdMessage
from fit_tool.profile.messages.record_message import RecordMessage
from fit_tool.profile.profile_type import FileType, Manufacturer, Event, EventType
from fit_tool.record import Record
from fit_tool.utils.crc import crc16


class TestWriteActivityFile(unittest.TestCase):

    def setUp(self):
        super().setUp()

    def shortDescription(self):
        return None

    def test_write_activity_file_iteratively(self):
        """Test for writing activity file iteratively
        """
        start_timestamp_millis = round(datetime.datetime(2022, 5, 10, 5, 5, 5).timestamp()) * 1000
        timestamp = start_timestamp_millis

        records_crc = 0
        records_size = 0

        def write_message(message_: Message):
            """Helper method to write message to a file while accumulating records size and records crc"""
            nonlocal records_crc
            nonlocal records_size

            record_ = Record.from_message(message_)
            buffer_ = record_.to_bytes()
            file_object.write(buffer_)
            records_size += len(buffer_)
            records_crc = crc16(buffer_, crc=records_crc)

        out_path = os.path.join(TEST_DIR, 'out', 'activity_iterative.fit')
        with open(out_path, 'wb') as file_object:
            # Create a placeholder 14 byte header (includes a crc). We overwrite these bytes after all the records
            # have been written.
            header = FitFileHeader(records_size=0, crc=0)
            file_object.write(header.to_bytes())

            message = FileIdMessage()
            message.type = FileType.ACTIVITY
            message.manufacturer = Manufacturer.STAGES_CYCLING
            message.product = 1
            message.time_created = timestamp
            message.serial_number = 0x12345678
            definition = DefinitionMessage.from_data_message(message)
            write_message(definition)
            write_message(message)

            # It is a best practice to include timer start and stop events in all Activity files. A timer start event
            # should occur before the first Record message in the file, and a timer stop event should occur after the
            # last Record message in the file when the activity recording is complete. Timer stop and start events
            # should be used anytime the activity recording has been paused and resumed. Record messages should not be
            # encoded to the file when the timer is paused.
            message = EventMessage()
            message.event = Event.TIMER
            message.event_type = EventType.START
            message.timestamp = timestamp
            definition = DefinitionMessage.from_data_message(message)
            write_message(definition)
            write_message(message)

            timestamp += 1000
            distance = 0

            message = RecordMessage()
            message.position_lat = 40.0
            message.position_long = -105.2613892
            message.distance = distance
            message.timestamp = timestamp
            definition = DefinitionMessage.from_data_message(message)
            write_message(definition)  # write the definition only once for all the record messages
            write_message(message)

            distance += 5
            timestamp += 1000
            message = RecordMessage()
            message.position_lat = 40.0
            message.position_long = -105.1613892
            message.distance = distance
            message.timestamp = timestamp
            write_message(message)

            message = EventMessage()
            message.event = Event.TIMER
            message.event_type = EventType.STOP
            message.timestamp = timestamp
            definition = DefinitionMessage.from_data_message(message)
            write_message(definition)
            write_message(message)

            # write crc
            buffer = struct.pack('<H', records_crc)
            file_object.write(buffer)

            # inject updated FIT file header at start of file with a header crc
            header = FitFileHeader(records_size=records_size, gen_crc=True)
            file_object.seek(0)
            file_object.write(header.to_bytes())

        # read back the file
        with open(out_path, 'rb') as file_object:
            bytes_buffer = file_object.read()
            fit_file = FitFile.from_bytes(bytes_buffer)
            # for row in fit_file.to_rows():
            #     print(row)
            record_message_index = 0
            for record in fit_file.records:
                message = record.message
                if isinstance(message, RecordMessage):
                    self.assertEqual(message.distance, record_message_index * 5)
                    record_message_index += 1

    def test_write_activity_with_developer_data_fields(self):
        """Test encoding an activity with developer data fields.
        """
        # Set auto_define to true, so that the builder creates the required Definition Messages for us.
        builder = FitFileBuilder(auto_define=True, min_string_size=50)

        message = FileIdMessage()
        message.type = FileType.ACTIVITY
        message.manufacturer = Manufacturer.DEVELOPMENT.value
        message.product = 1231
        message.time_created = round(datetime.datetime.now().timestamp() * 1000)
        message.serial_number = 0x12345678
        builder.add(message)

        developer_data_index = 0
        message = DeveloperDataIdMessage()
        message.application_id = b'123456'
        message.developer_data_index = developer_data_index
        builder.add(message)

        # Define the developer field
        doughnuts_earned_field_name = 'doughnuts_earned'
        doughnuts_earned_field_units = 'doughnuts'
        message = FieldDescriptionMessage()
        message.developer_data_index = developer_data_index
        message.field_definition_number = 0
        message.fit_base_type_id = BaseType.SINT8
        message.field_name = doughnuts_earned_field_name
        message.units = doughnuts_earned_field_units
        builder.add(message)

        # Add a record message with an addition doughnuts earned developer field
        dev_field = DeveloperField(developer_data_index=developer_data_index, field_id=0, size=1)
        dev_field.set_value(0, 5)

        message = RecordMessage(developer_fields=[dev_field])
        message.distance = 0
        message.power = 100
        builder.add(message)

        # Finally build the FIT file object and write it to a file
        fit_file = builder.build()

        out_path = os.path.join(TEST_DIR, 'out', 'activity_with_developer_data.fit')

        fit_file.to_file(out_path)
        csv_path = os.path.join(TEST_DIR, 'out', 'activity_with_developer_data.csv')
        fit_file.to_csv(csv_path)

        # read back the file
        with open(out_path, 'rb') as file_object:
            bytes_buffer = file_object.read()
            fit_file = FitFile.from_bytes(bytes_buffer)
            for row in fit_file.to_rows():
                print(row)
            for record in fit_file.records:
                message = record.message
                if isinstance(message, RecordMessage):
                    self.assertEqual(message.developer_fields[0].name, doughnuts_earned_field_name)
                    self.assertEqual(message.developer_fields[0].get_value(0), 5)
                    self.assertEqual(message.developer_fields[0].units, doughnuts_earned_field_units)



import datetime

from fit_tool.fit_file import FitFile
from fit_tool.fit_file_builder import FitFileBuilder
from fit_tool.profile.messages.file_id_message import FileIdMessage
from fit_tool.profile.messages.workout_message import WorkoutMessage
from fit_tool.profile.messages.workout_step_message import WorkoutStepMessage
from fit_tool.profile.profile_type import Sport, Intensity, WorkoutStepDuration, WorkoutStepTarget, Manufacturer, \
    FileType


def main():
    file_id_message = FileIdMessage()
    file_id_message.type = FileType.WORKOUT
    file_id_message.manufacturer = Manufacturer.DEVELOPMENT.value
    file_id_message.product = 0
    file_id_message.time_created = round(datetime.datetime.now().timestamp() * 1000)
    file_id_message.serial_number = 0x12345678

    workout_steps = []
    step = WorkoutStepMessage()
    step.workout_step_name = 'Warm up 10min in Heart Rate Zone 1'
    step.intensity = Intensity.WARMUP
    step.duration_type = WorkoutStepDuration.TIME
    step.duration_time = 600.0
    step.target_type = WorkoutStepTarget.HEART_RATE
    step.target_hr_zone = 1
    workout_steps.append(step)

    step = WorkoutStepMessage()
    step.workout_step_name = 'Bike 40min Power Zone 3'
    step.intensity = Intensity.ACTIVE
    step.duration_type = WorkoutStepDuration.TIME
    step.duration_time = 24000.0
    step.target_type = WorkoutStepTarget.POWER
    step.target_power_zone = 3
    workout_steps.append(step)

    step = WorkoutStepMessage()
    step.workout_step_name = 'Cool Down Until Lap Button Pressed'
    step.intensity = Intensity.COOLDOWN
    step.duration_type = WorkoutStepDuration.OPEN
    step.durationValue = 0
    step.target_type = WorkoutStepTarget.OPEN
    step.target_value = 0
    workout_steps.append(step)

    workout_message = WorkoutMessage()
    workout_message.workoutName = 'Tempo Bike'
    workout_message.sport = Sport.CYCLING
    workout_message.num_valid_steps = len(workout_steps)

    # We set autoDefine to true, so that the builder creates the required
    # Definition Messages for us.
    builder = FitFileBuilder(auto_define=True, min_string_size=50)
    builder.add(file_id_message)
    builder.add(workout_message)
    builder.add_all(workout_steps)

    fit_file = builder.build()

    fit_file.to_file('../tests/out/tempo_bike_workout.fit')

    fit_file2 = FitFile.from_file('../tests/out/tempo_bike_workout.fit')
    fit_file2.to_csv('../tests/out/tempo_bike_workout.csv')


if __name__ == "__main__":
    main()



# nosetests --nocapture  tests/test_field.py
import csv
import os
import unittest

TEST_DIR = os.path.dirname(__file__)
OUT_DIR = os.path.join(TEST_DIR, 'out')
os.makedirs(OUT_DIR, exist_ok=True)

from fit_tool.fit_file import FitFile


class TestSdkFiles(unittest.TestCase):

    def setUp(self):
        super().setUp()
        # stream_handler = logging.StreamHandler(sys.stdout)
        # stream_handler.setFormatter(formatter)
        # logger.addHandler(stream_handler)

    def shortDescription(self):
        return None

    def test_developer_data(self):
        """Test decoding activity file with developer fields.
        """
        path = os.path.join(os.path.dirname(__file__), 'data/sdk/DeveloperData.fit')

        with open(path, 'rb') as file_object:
            bytes_buffer = file_object.read()
            fit_file = FitFile.from_bytes(bytes_buffer)
            # print(f'Profile version: {fit_file.header.profile_version}')
            fit_file.to_rows()

    def test_activity(self):
        """Test decoding activity file.
        """
        path = os.path.join(os.path.dirname(__file__), 'data/sdk/Activity.fit')

        with open(path, 'rb') as file_object:
            bytes_buffer = file_object.read()
            fit_file = FitFile.from_bytes(bytes_buffer)
            # print(f'Profile version: {fit_file.header.profile_version}')
            rows = fit_file.to_rows()

        out_path = os.path.join(os.path.dirname(__file__), 'out/Activity.csv')
        with open(out_path, 'w', newline='') as csv_file:
            csv_writer = csv.writer(csv_file, delimiter=',',
                                    quotechar='|', quoting=csv.QUOTE_MINIMAL)
            for row in rows:
                csv_writer.writerow(row)

    def test_activity_developer_data(self):
        """Test decoding activity file.
        """
        path = os.path.join(os.path.dirname(__file__), 'data/sdk/activity_developerdata.fit')

        with open(path, 'rb') as file_object:
            bytes_buffer = file_object.read()
            fit_file = FitFile.from_bytes(bytes_buffer)
            # print(f'Profile version: {fit_file.header.profile_version}')
            rows = fit_file.to_rows()

        out_path = os.path.join(os.path.dirname(__file__), 'out/activity_developerdata.csv')
        with open(out_path, 'w', newline='') as csv_file:
            csv_writer = csv.writer(csv_file, delimiter=',',
                                    quotechar='|', quoting=csv.QUOTE_MINIMAL)
            for row in rows:
                csv_writer.writerow(row)

    def test_activity_low_battery(self):
        """Test decoding activity low battery file.
        """
        path = os.path.join(os.path.dirname(__file__), 'data/sdk/activity_lowbattery.fit')

        with open(path, 'rb') as file_object:
            bytes_buffer = file_object.read()
            fit_file = FitFile.from_bytes(bytes_buffer)
            # print(f'Profile version: {fit_file.header.profile_version}')
            fit_file.to_rows()

    def test_activity_multi_sport(self):
        """Test decoding activity multisport file.
        """
        path = os.path.join(os.path.dirname(__file__), 'data/sdk/activity_multisport.fit')

        with open(path, 'rb') as file_object:
            bytes_buffer = file_object.read()
            fit_file = FitFile.from_bytes(bytes_buffer)
            # print(f'Profile version: {fit_file.header.profile_version}')
            fit_file.to_rows()

    def test_activity_poolswim_with_hr(self):
        """Test decoding
        """
        path = os.path.join(os.path.dirname(__file__), 'data/sdk/activity_poolswim_with_hr.fit')

        with open(path, 'rb') as file_object:
            bytes_buffer = file_object.read()
            fit_file = FitFile.from_bytes(bytes_buffer)
            # print(f'Profile version: {fit_file.header.profile_version}')
            fit_file.to_rows()

    def test_activity_poolswim(self):
        """Test decoding
        """
        path = os.path.join(os.path.dirname(__file__), 'data/sdk/activity_poolswim.fit')

        with open(path, 'rb') as file_object:
            bytes_buffer = file_object.read()
            fit_file = FitFile.from_bytes(bytes_buffer)
            # print(f'Profile version: {fit_file.header.profile_version}')
            fit_file.to_rows()

    def test_activity_truncated(self):
        """Test decoding
        """
        path = os.path.join(os.path.dirname(__file__), 'data/sdk/activity_truncated.fit')

        with open(path, 'rb') as file_object:
            bytes_buffer = file_object.read()
            fit_file = FitFile.from_bytes(bytes_buffer)
            # print(f'Profile version: {fit_file.header.profile_version}')
            fit_file.to_rows()

    def test_monitoring_file(self):
        """Test decoding
        """
        path = os.path.join(os.path.dirname(__file__), 'data/sdk/MonitoringFile.fit')

        with open(path, 'rb') as file_object:
            bytes_buffer = file_object.read()
            fit_file = FitFile.from_bytes(bytes_buffer)
            # print(f'Profile version: {fit_file.header.profile_version}')
            fit_file.to_rows()

    def test_weight_scale_multi_user(self):
        """Test decoding
        """
        path = os.path.join(os.path.dirname(__file__), 'data/sdk/WeightScaleMultiUser.fit')

        with open(path, 'rb') as file_object:
            bytes_buffer = file_object.read()
            fit_file = FitFile.from_bytes(bytes_buffer)
            # print(f'Profile version: {fit_file.header.profile_version}')
            fit_file.to_rows()

    def test_weight_scale_single_user(self):
        """Test decoding
        """
        path = os.path.join(os.path.dirname(__file__), 'data/sdk/WeightScaleSingleUser.fit')

        with open(path, 'rb') as file_object:
            bytes_buffer = file_object.read()
            fit_file = FitFile.from_bytes(bytes_buffer)
            # print(f'Profile version: {fit_file.header.profile_version}')
            fit_file.to_rows()

    def test_workout_individual_steps(self):
        """Test decoding
        """
        path = os.path.join(os.path.dirname(__file__), 'data/sdk/WorkoutIndividualSteps.fit')

        with open(path, 'rb') as file_object:
            bytes_buffer = file_object.read()
            fit_file = FitFile.from_bytes(bytes_buffer)
            # print(f'Profile version: {fit_file.header.profile_version}')
            fit_file.to_rows()

    def test_workout_custom_target_values(self):
        """Test decoding
        """
        path = os.path.join(os.path.dirname(__file__), 'data/sdk/WorkoutCustomTargetValues.fit')

        with open(path, 'rb') as file_object:
            bytes_buffer = file_object.read()
            fit_file = FitFile.from_bytes(bytes_buffer)
            # print(f'Profile version: {fit_file.header.profile_version}')
            fit_file.to_rows()

    def test_workout_repeat_greater_than_step(self):
        """Test decoding
        """
        path = os.path.join(os.path.dirname(__file__), 'data/sdk/WorkoutRepeatGreaterThanStep.fit')

        with open(path, 'rb') as file_object:
            bytes_buffer = file_object.read()
            fit_file = FitFile.from_bytes(bytes_buffer)
            # print(f'Profile version: {fit_file.header.profile_version}')
            fit_file.to_rows()

    def test_workout_repeat_steps(self):
        """Test decoding workout repeat steps file.
        """
        path = os.path.join(os.path.dirname(__file__), 'data/sdk/WorkoutRepeatSteps.fit')

        with open(path, 'rb') as file_object:
            bytes_buffer = file_object.read()
            fit_file = FitFile.from_bytes(bytes_buffer)
            # print(f'Profile version: {fit_file.header.profile_version}')
            fit_file.to_rows()



import datetime

import gpxpy
from geopy.distance import geodesic

from fit_tool.fit_file_builder import FitFileBuilder
from fit_tool.profile.messages.course_message import CourseMessage
from fit_tool.profile.messages.course_point_message import CoursePointMessage
from fit_tool.profile.messages.event_message import EventMessage
from fit_tool.profile.messages.file_id_message import FileIdMessage
from fit_tool.profile.messages.lap_message import LapMessage
from fit_tool.profile.messages.record_message import RecordMessage
from fit_tool.profile.profile_type import FileType, Manufacturer, Sport, Event, EventType, CoursePoint


def main():
    # Set auto_define to true, so that the builder creates the required Definition Messages for us.
    builder = FitFileBuilder(auto_define=True, min_string_size=50)

    # Read position data from a GPX file
    gpx_file = open('../tests/data/old_stage_left_hand_lee.gpx', 'r')
    gpx = gpxpy.parse(gpx_file)

    message = FileIdMessage()
    message.type = FileType.COURSE
    message.manufacturer = Manufacturer.DEVELOPMENT.value
    message.product = 0
    message.timeCreated = round(datetime.datetime.now().timestamp() * 1000)
    message.serialNumber = 0x12345678
    builder.add(message)

    # Every FIT course file MUST contain a Course message
    message = CourseMessage()
    message.courseName = 'old stage'
    message.sport = Sport.CYCLING
    builder.add(message)

    # Timer Events are REQUIRED for FIT course files
    start_timestamp = round(datetime.datetime.now().timestamp() * 1000)
    message = EventMessage()
    message.event = Event.TIMER
    message.event_type = EventType.START
    message.timestamp = start_timestamp
    builder.add(message)

    distance = 0.0
    timestamp = start_timestamp

    course_records = []  # track points

    prev_coordinate = None

    for track_point in gpx.tracks[0].segments[0].points:
        current_coordinate = (track_point.latitude, track_point.longitude)

        # calculate distance from previous coordinate and accumulate distance
        if prev_coordinate:
            delta = geodesic(prev_coordinate, current_coordinate).meters
        else:
            delta = 0.0
        distance += delta

        message = RecordMessage()
        message.position_lat = track_point.latitude
        message.position_long = track_point.longitude
        message.distance = distance
        message.timestamp = timestamp
        course_records.append(message)

        timestamp += 10000
        prev_coordinate = current_coordinate

    builder.add_all(course_records)

    #  Add start and end course points (i.e. way points)
    #
    message = CoursePointMessage()
    message.timestamp = course_records[0].timestamp
    message.position_lat = course_records[0].position_lat
    message.position_long = course_records[0].position_long
    message.type = CoursePoint.SEGMENT_START
    message.course_point_name = 'start'
    builder.add(message)

    message = CoursePointMessage()
    message.timestamp = course_records[-1].timestamp
    message.position_lat = course_records[-1].position_lat
    message.position_long = course_records[-1].position_long
    message.type = CoursePoint.SEGMENT_END
    message.course_point_name = 'end'
    builder.add(message)

    # stop event
    message = EventMessage()
    message.event = Event.TIMER
    message.eventType = EventType.STOP_ALL
    message.timestamp = timestamp
    builder.add(message)

    # Every FIT course file MUST contain a Lap message
    elapsed_time = timestamp - start_timestamp
    message = LapMessage()
    message.timestamp = timestamp
    message.start_time = start_timestamp
    message.total_elapsed_time = elapsed_time/1000   # seconds
    message.total_timer_time = elapsed_time/1000     # seconds
    message.start_position_lat = course_records[0].position_lat
    message.start_position_long = course_records[0].position_long
    message.end_position_lat = course_records[-1].position_lat
    message.endPositionLong = course_records[-1].position_long
    message.total_distance = course_records[-1].distance
    builder.add(message)

    # Finally build the FIT file object and write it to a file
    fit_file = builder.build()

    out_path = '../tests/out/old_stage_course.fit'
    fit_file.to_file(out_path)
    csv_path = '../tests/out/old_stage_course.csv'
    fit_file.to_csv(csv_path)


if __name__ == "__main__":
    main()



# nosetests --nocapture  tests/test_fit_file.py


import unittest

from fit_tool.definition_message import DefinitionMessage
from fit_tool.fit_file import FitFile
from fit_tool.fit_file_builder import FitFileBuilder
from fit_tool.profile.messages.workout_step_message import WorkoutStepMessage
from fit_tool.profile.profile_type import WorkoutStepDuration


class TestFitFile(unittest.TestCase):

    def test_conversion_simple(self):
        mesg = WorkoutStepMessage(local_id=0)
        mesg.workout_step_name = '1st step'
        mesg.duration_type = WorkoutStepDuration.DISTANCE

        def_mesg = DefinitionMessage.from_data_message(mesg)

        builder = FitFileBuilder(auto_define=False)
        builder.add(def_mesg)
        builder.add(mesg)

        fit_file = builder.build()

        bytes1 = fit_file.to_bytes()

        fite_file2 = FitFile.from_bytes(bytes1)
        bytes2 = fite_file2.to_bytes()

        print(f'{bytes1}')
        print(f'{bytes2}')
        self.assertEqual(bytes2, bytes1)

    def test_builder_with_auto_define(self):
        mesg1 = WorkoutStepMessage(local_id=0)
        mesg1.workout_step_name = '1st step'
        mesg1.duration_type = WorkoutStepDuration.DISTANCE

        mesg2 = WorkoutStepMessage(local_id=0)
        mesg2.workout_step_name = '2nd step'
        mesg2.duration_type = WorkoutStepDuration.DISTANCE

        builder = FitFileBuilder(auto_define=True, min_string_size=50)
        builder.add(mesg1)
        builder.add(mesg2)

        fit_file = builder.build()

        self.assertEqual(len(fit_file.records), 3)



"""
Command line interface for fit_tool.

Copyright (c) 2017 Stages Cycling. All rights reserved.
"""
import argparse
import logging
import os

from fit_tool.fit_file import FitFile
from fit_tool.utils.logging import logger


def parse_args():
    """Parse command line arguments."""
    parser = argparse.ArgumentParser(
        description="Tool for managing FIT files."
    )
    parser.add_argument(
        'fitfile',
        metavar='FILE',
        help='FIT file to process'
    )
    parser.add_argument(
        '-v', '--verbose',
        action='store_true',
        help='specify verbose output'
    )
    parser.add_argument("-o", "--output", help="Output filename.")
    parser.add_argument("-l", "--log", help="Log filename.")
    parser.add_argument(
        "-t", "--type",
        help="Output format type. Options: csv, fit."
    )

    return parser.parse_args()


def main():
    """Main entry point."""
    args = parse_args()

    formatter = logging.Formatter(fmt="%(asctime)s %(levelname)s %(message)s")

    if args.log:
        handler = logging.FileHandler(args.log)
        handler.setFormatter(formatter)
        logger.addHandler(handler)

    if args.verbose:
        handler = logging.StreamHandler()
        handler.setFormatter(formatter)
        logger.addHandler(handler)
        logger.setLevel(logging.DEBUG)
    logger.info(f'Loading fit file {args.fitfile}...')

    fit_file = FitFile.from_file(args.fitfile)

    if args.type:
        format_type = args.type
    elif args.output:
        _, out_ext = os.path.splitext(os.path.basename(args.output))
        format_type = out_ext.lstrip('.')
    else:
        format_type = 'csv'

    basename_noext, _ = os.path.splitext(os.path.basename(args.fitfile))
    output_filename = args.output or f'{basename_noext}.{format_type}'

    logger.info(f'Exporting fit file to {output_filename} as format {format_type}...')

    if format_type == 'csv':
        fit_file.to_csv(output_filename)
    elif format_type == 'fit':
        fit_file.to_file(output_filename)


if __name__ == "__main__":
    main()



# nosetests --nocapture  tests/test_field.py

import unittest

from fit_tool.base_type import BaseType
from fit_tool.field import Field
from fit_tool.field_definition import FieldDefinition


class TestField(unittest.TestCase):

    def shortDescription(self):
        return None

    def test_field_definition_conversions(self):
        fd1 = FieldDefinition(field_id=255, size=20, base_type=BaseType.UINT16)
        bytes1 = fd1.to_bytes()
        fd2 = FieldDefinition.from_bytes(bytes1)
        bytes2 = fd2.to_bytes()

        self.assertEqual(bytes2, bytes1)

    def test_field_integer_conversions(self):

        for base_type in BaseType:
            if base_type.is_float() or base_type.is_big() or base_type.is_string:
                continue

            field = Field(base_type=base_type)
            min_value = base_type.min
            max_value = base_type.max
            if min_value is not None and max_value is not None:
                value = min_value
                bytes_buffer = field.encoded_value_to_bytes(value)
                value_from_bytes = field.get_encoded_value_from_bytes(bytes_buffer);
                self.assertEqual(value_from_bytes, value)

                value = max_value
                bytes_buffer = field.encoded_value_to_bytes(value)
                value_from_bytes = field.get_encoded_value_from_bytes(bytes_buffer)
                self.assertEqual(value_from_bytes, value)

    def test_field_string_conversions(self):
        field = Field(base_type=BaseType.STRING)
        value = 'test12345'
        bytes_buffer = field.encoded_value_to_bytes(value)
        value_from_bytes = field.get_encoded_value_from_bytes(bytes_buffer)
        self.assertEqual(value, value_from_bytes)

    def test_field_float_conversions(self):
        field = Field(base_type=BaseType.FLOAT32)
        value = 3.14
        bytes_buffer = field.encoded_value_to_bytes(value)
        value_from_bytes = field.get_encoded_value_from_bytes(bytes_buffer)
        bytes2 = field.encoded_value_to_bytes(value_from_bytes)

        self.assertAlmostEqual(value, value_from_bytes, 3)
        self.assertEqual(bytes2, bytes_buffer)

        field = Field(base_type=BaseType.FLOAT64)
        value = 3.14
        bytes_buffer = field.encoded_value_to_bytes(value)
        value_from_bytes = field.get_encoded_value_from_bytes(bytes_buffer)
        bytes2 = field.encoded_value_to_bytes(value_from_bytes)

        self.assertAlmostEqual(value, value_from_bytes, 3)
        self.assertEqual(bytes2, bytes_buffer)

    def test_field_string_to_row(self):
        field = Field(name='title', base_type=BaseType.STRING, growable=True)
        value = 'test12345'
        field.set_encoded_value(0, value)
        field.to_row()



# Autogenerated. Do not modify.
#
# Profile: 21.60
from typing import List as list
from typing import Optional

from fit_tool.base_type import BaseType
from fit_tool.data_message import DataMessage
from fit_tool.definition_message import DefinitionMessage
from fit_tool.developer_field import DeveloperField
from fit_tool.endian import Endian
from fit_tool.field import Field
from fit_tool.profile.profile_type import *
from fit_tool.sub_field import SubField


class ThreeDSensorCalibrationMessage(DataMessage):
    ID = 167
    NAME = 'three_d_sensor_calibration'

    @staticmethod
    def __get_field_size(definition_message: DefinitionMessage, field_id: int) -> int:
        size = 0
        if definition_message:
            field_definition = definition_message.get_field_definition(field_id)
            if field_definition:
                size = field_definition.size

        return size

    def __init__(self, definition_message=None, developer_fields=None, local_id: int = 0,
                 endian: Endian = Endian.LITTLE):
        super().__init__(name=ThreeDSensorCalibrationMessage.NAME,
                         global_id=ThreeDSensorCalibrationMessage.ID,
                         local_id=definition_message.local_id if definition_message else local_id,
                         endian=definition_message.endian if definition_message else endian,
                         definition_message=definition_message,
                         developer_fields=developer_fields,
                         fields=[
                             TimestampField(
                                 size=self.__get_field_size(definition_message, TimestampField.ID),
                                 growable=definition_message is None),
                             ThreeDSensorCalibrationSensorTypeField(
                                 size=self.__get_field_size(definition_message,
                                                            ThreeDSensorCalibrationSensorTypeField.ID),
                                 growable=definition_message is None),
                             ThreeDSensorCalibrationCalibrationFactorField(
                                 size=self.__get_field_size(definition_message,
                                                            ThreeDSensorCalibrationCalibrationFactorField.ID),
                                 growable=definition_message is None),
                             ThreeDSensorCalibrationCalibrationDivisorField(
                                 size=self.__get_field_size(definition_message,
                                                            ThreeDSensorCalibrationCalibrationDivisorField.ID),
                                 growable=definition_message is None),
                             ThreeDSensorCalibrationLevelShiftField(
                                 size=self.__get_field_size(definition_message,
                                                            ThreeDSensorCalibrationLevelShiftField.ID),
                                 growable=definition_message is None),
                             ThreeDSensorCalibrationOffsetCalField(
                                 size=self.__get_field_size(definition_message,
                                                            ThreeDSensorCalibrationOffsetCalField.ID),
                                 growable=definition_message is None),
                             ThreeDSensorCalibrationOrientationMatrixField(
                                 size=self.__get_field_size(definition_message,
                                                            ThreeDSensorCalibrationOrientationMatrixField.ID),
                                 growable=definition_message is None)
                         ])

        self.growable = self.definition_message is None

    @classmethod
    def from_bytes(cls, definition_message: DefinitionMessage, developer_fields: list[DeveloperField],
                   bytes_buffer: bytes, offset: int = 0):
        message = cls(definition_message=definition_message, developer_fields=developer_fields)
        message.read_from_bytes(bytes_buffer, offset)
        return message

    # timestamp : milliseconds from January 1st, 1970 at 00:00:00 UTC

    @property
    def timestamp(self) -> Optional[int]:
        field = self.get_field(TimestampField.ID)
        if field and field.is_valid():
            sub_field = field.get_valid_sub_field(self.fields)
            return field.get_value(sub_field=sub_field)
        else:
            return None

    # timestamp : milliseconds from January 1st, 1970 at 00:00:00 UTC

    @timestamp.setter
    def timestamp(self, value: int):
        field = self.get_field(TimestampField.ID)

        if field:
            if value is None:
                field.clear()
            else:
                sub_field = field.get_valid_sub_field(self.fields)
                field.set_value(0, value, sub_field)

    @property
    def sensor_type(self) -> Optional[SensorType]:
        field = self.get_field(ThreeDSensorCalibrationSensorTypeField.ID)
        if field and field.is_valid():
            sub_field = field.get_valid_sub_field(self.fields)
            return field.get_value(sub_field=sub_field)
        else:
            return None

    @sensor_type.setter
    def sensor_type(self, value: SensorType):
        field = self.get_field(ThreeDSensorCalibrationSensorTypeField.ID)

        if field:
            if value is None:
                field.clear()
            else:
                sub_field = field.get_valid_sub_field(self.fields)
                field.set_value(0, value, sub_field)

    @property
    def calibration_factor(self) -> Optional[int]:
        field = self.get_field(ThreeDSensorCalibrationCalibrationFactorField.ID)
        if field and field.is_valid():
            sub_field = field.get_valid_sub_field(self.fields)
            return field.get_value(sub_field=sub_field)
        else:
            return None

    @calibration_factor.setter
    def calibration_factor(self, value: int):
        field = self.get_field(ThreeDSensorCalibrationCalibrationFactorField.ID)

        if field:
            if value is None:
                field.clear()
            else:
                sub_field = field.get_valid_sub_field(self.fields)
                field.set_value(0, value, sub_field)

    @property
    def accel_cal_factor(self) -> Optional[int]:
        field = self.get_field(ThreeDSensorCalibrationCalibrationFactorField.ID)
        type_field = self.get_field(ThreeDSensorCalibrationSensorTypeField.ID)

        is_sub_field_valid = type_field and type_field.get_value() in [0]
        if field and field.is_valid() and is_sub_field_valid:
            sub_field = field.get_valid_sub_field(self.fields)
            return field.get_value(sub_field=sub_field)
        else:
            return None

    @accel_cal_factor.setter
    def accel_cal_factor(self, value: int):
        field = self.get_field(ThreeDSensorCalibrationCalibrationFactorField.ID)
        if field:
            if value is None:
                field.clear()
            else:
                sub_field = field.get_valid_sub_field(self.fields)
                field.set_value(0, value, sub_field)

    @property
    def gyro_cal_factor(self) -> Optional[int]:
        field = self.get_field(ThreeDSensorCalibrationCalibrationFactorField.ID)
        type_field = self.get_field(ThreeDSensorCalibrationSensorTypeField.ID)

        is_sub_field_valid = type_field and type_field.get_value() in [1]
        if field and field.is_valid() and is_sub_field_valid:
            sub_field = field.get_valid_sub_field(self.fields)
            return field.get_value(sub_field=sub_field)
        else:
            return None

    @gyro_cal_factor.setter
    def gyro_cal_factor(self, value: int):
        field = self.get_field(ThreeDSensorCalibrationCalibrationFactorField.ID)
        if field:
            if value is None:
                field.clear()
            else:
                sub_field = field.get_valid_sub_field(self.fields)
                field.set_value(0, value, sub_field)

    @property
    def calibration_divisor(self) -> Optional[int]:
        field = self.get_field(ThreeDSensorCalibrationCalibrationDivisorField.ID)
        if field and field.is_valid():
            sub_field = field.get_valid_sub_field(self.fields)
            return field.get_value(sub_field=sub_field)
        else:
            return None

    @calibration_divisor.setter
    def calibration_divisor(self, value: int):
        field = self.get_field(ThreeDSensorCalibrationCalibrationDivisorField.ID)

        if field:
            if value is None:
                field.clear()
            else:
                sub_field = field.get_valid_sub_field(self.fields)
                field.set_value(0, value, sub_field)

    @property
    def level_shift(self) -> Optional[int]:
        field = self.get_field(ThreeDSensorCalibrationLevelShiftField.ID)
        if field and field.is_valid():
            sub_field = field.get_valid_sub_field(self.fields)
            return field.get_value(sub_field=sub_field)
        else:
            return None

    @level_shift.setter
    def level_shift(self, value: int):
        field = self.get_field(ThreeDSensorCalibrationLevelShiftField.ID)

        if field:
            if value is None:
                field.clear()
            else:
                sub_field = field.get_valid_sub_field(self.fields)
                field.set_value(0, value, sub_field)

    @property
    def offset_cal(self) -> Optional[list[int]]:
        field = self.get_field(ThreeDSensorCalibrationOffsetCalField.ID)
        if field and field.is_valid():
            return field.get_values()
        else:
            return None

    @offset_cal.setter
    def offset_cal(self, value: list[int]):
        field = self.get_field(ThreeDSensorCalibrationOffsetCalField.ID)

        if field:
            if value is None:
                field.clear()
            else:
                field.set_values(value)

    @property
    def orientation_matrix(self) -> Optional[list[float]]:
        field = self.get_field(ThreeDSensorCalibrationOrientationMatrixField.ID)
        if field and field.is_valid():
            return field.get_values()
        else:
            return None

    @orientation_matrix.setter
    def orientation_matrix(self, value: list[float]):
        field = self.get_field(ThreeDSensorCalibrationOrientationMatrixField.ID)

        if field:
            if value is None:
                field.clear()
            else:
                field.set_values(value)


class TimestampField(Field):
    ID = 253

    def __init__(self, size: int = 0, growable: bool = True):
        super().__init__(
            name='timestamp',
            field_id=self.ID,
            base_type=BaseType.UINT32,
            offset=-631065600000,
            scale=0.001,
            size=size,
            units='ms',
            type_name='date_time',
            growable=growable,
            sub_fields=[
            ]
        )


class ThreeDSensorCalibrationSensorTypeField(Field):
    ID = 0

    def __init__(self, size: int = 0, growable: bool = True):
        super().__init__(
            name='sensor_type',
            field_id=self.ID,
            base_type=BaseType.ENUM,
            offset=0,
            scale=1,
            size=size,
            growable=growable,
            sub_fields=[
            ]
        )


class ThreeDSensorCalibrationCalibrationFactorField(Field):
    ID = 1

    def __init__(self, size: int = 0, growable: bool = True):
        super().__init__(
            name='calibration_factor',
            field_id=self.ID,
            base_type=BaseType.UINT32,
            offset=0,
            scale=1,
            size=size,
            growable=growable,
            sub_fields=[
                SubField(
                    name='accel_cal_factor',
                    base_type=BaseType.UINT32,
                    scale=1,
                    offset=0,
                    units='g',
                    reference_map={
                        ThreeDSensorCalibrationSensorTypeField.ID: [0]
                    }),
                SubField(
                    name='gyro_cal_factor',
                    base_type=BaseType.UINT32,
                    scale=1,
                    offset=0,
                    units='deg/s',
                    reference_map={
                        ThreeDSensorCalibrationSensorTypeField.ID: [1]
                    })
            ]
        )


class ThreeDSensorCalibrationCalibrationDivisorField(Field):
    ID = 2

    def __init__(self, size: int = 0, growable: bool = True):
        super().__init__(
            name='calibration_divisor',
            field_id=self.ID,
            base_type=BaseType.UINT32,
            offset=0,
            scale=1,
            size=size,
            units='counts',
            type_name='',
            growable=growable,
            sub_fields=[
            ]
        )


class ThreeDSensorCalibrationLevelShiftField(Field):
    ID = 3

    def __init__(self, size: int = 0, growable: bool = True):
        super().__init__(
            name='level_shift',
            field_id=self.ID,
            base_type=BaseType.UINT32,
            offset=0,
            scale=1,
            size=size,
            growable=growable,
            sub_fields=[
            ]
        )


class ThreeDSensorCalibrationOffsetCalField(Field):
    ID = 4

    def __init__(self, size: int = 0, growable: bool = True):
        super().__init__(
            name='offset_cal',
            field_id=self.ID,
            base_type=BaseType.SINT32,
            offset=0,
            scale=1,
            size=size,
            growable=growable,
            sub_fields=[
            ]
        )


class ThreeDSensorCalibrationOrientationMatrixField(Field):
    ID = 5

    def __init__(self, size: int = 0, growable: bool = True):
        super().__init__(
            name='orientation_matrix',
            field_id=self.ID,
            base_type=BaseType.SINT32,
            offset=0,
            scale=65535,
            size=size,
            growable=growable,
            sub_fields=[
            ]
        )



# nosetests --nocapture  tests/test_field.py

import unittest

from fit_tool.base_type import BaseType
from fit_tool.definition_message import DefinitionMessage
from fit_tool.endian import Endian
from fit_tool.field_definition import FieldDefinition
from fit_tool.profile.messages.workout_step_message import WorkoutStepMessage
from fit_tool.profile.profile_type import WorkoutStepDuration


class TestDefinitionMessage(unittest.TestCase):

    def shortDescription(self):
        return None

    def test_definition_message_conversions(self):
        dm1 = DefinitionMessage(global_id=555, local_id=20,
                                field_definitions=[FieldDefinition(field_id=100, size=5, base_type=BaseType.STRING)])
        bytes1 = dm1.to_bytes()
        dm2 = DefinitionMessage.from_bytes(bytes1)
        bytes2 = dm2.to_bytes()

        self.assertEqual(bytes2, bytes1)

    def test_big_endian_conversion(self):
        dm1 = DefinitionMessage(global_id=555, local_id=20,
                                endian=Endian.BIG,
                                field_definitions=[FieldDefinition(field_id=100, size=5, base_type=BaseType.STRING)])

        bytes1 = dm1.to_bytes()
        dm2 = DefinitionMessage.from_bytes(bytes1)
        bytes2 = dm2.to_bytes()

        self.assertEqual(bytes2, bytes1)
        self.assertEqual(dm2.endian, Endian.BIG)

    def test_to_row(self):
        dm1 = WorkoutStepMessage()
        dm1.workoutStepName = 'test'
        dm1.durationType = WorkoutStepDuration.DISTANCE

        definition = DefinitionMessage.from_data_message(dm1)
        row = definition.to_row()
        print(row)




import unittest
from fit_tool.data_message import DataMessage
from fit_tool.developer_field import DeveloperField

class TestDeveloperFields(unittest.TestCase):

    def setUp(self):
        self.message = DataMessage()
        self.message.developer_fields = [
            DeveloperField(field_id=1, developer_data_index=0, name='dev_field_1'),
            DeveloperField(field_id=2, developer_data_index=0, name='dev_field_2'),
            DeveloperField(field_id=1, developer_data_index=1, name='dev_field_3'),
        ]

    def test_get_developer_field_found(self):
        # Test finding first element
        field = self.message.get_developer_field(developer_data_index=0, field_id=1)
        self.assertIsNotNone(field)
        self.assertEqual(field.name, 'dev_field_1')

        # Test finding middle element
        field = self.message.get_developer_field(developer_data_index=0, field_id=2)
        self.assertIsNotNone(field)
        self.assertEqual(field.name, 'dev_field_2')

        # Test finding last element
        field = self.message.get_developer_field(developer_data_index=1, field_id=1)
        self.assertIsNotNone(field)
        self.assertEqual(field.name, 'dev_field_3')

    def test_get_developer_field_not_found(self):
        # Test non-existent field id
        field = self.message.get_developer_field(developer_data_index=0, field_id=999)
        self.assertIsNone(field)

        # Test non-existent developer data index
        field = self.message.get_developer_field(developer_data_index=99, field_id=1)
        self.assertIsNone(field)

    def test_get_developer_field_by_name_found(self):
        field = self.message.get_developer_field_by_name('dev_field_1')
        self.assertIsNotNone(field)
        self.assertEqual(field.developer_data_index, 0)
        self.assertEqual(field.field_id, 1)

    def test_get_developer_field_by_name_not_found(self):
        field = self.message.get_developer_field_by_name('non_existent_field')
        self.assertIsNone(field)



"""Tests for the command line interface."""

import os
import tempfile
import unittest
from unittest.mock import patch

from fit_tool.cli import main, parse_args


class TestParseArgs(unittest.TestCase):

    def test_parse_args_minimal(self):
        with patch('sys.argv', ['fit-tool', 'test.fit']):
            args = parse_args()
            self.assertEqual(args.fitfile, 'test.fit')
            self.assertFalse(args.verbose)
            self.assertIsNone(args.output)
            self.assertIsNone(args.log)
            self.assertIsNone(args.type)

    def test_parse_args_with_options(self):
        with patch('sys.argv', ['fit-tool', 'test.fit', '-v', '-o', 'out.csv', '-l', 'log.txt', '-t', 'csv']):
            args = parse_args()
            self.assertEqual(args.fitfile, 'test.fit')
            self.assertTrue(args.verbose)
            self.assertEqual(args.output, 'out.csv')
            self.assertEqual(args.log, 'log.txt')
            self.assertEqual(args.type, 'csv')


class TestMain(unittest.TestCase):

    def setUp(self):
        self.test_dir = tempfile.mkdtemp()
        self.test_fit_file = os.path.join(
            os.path.dirname(__file__),
            'data',
            'sdk',
            'Activity.fit'
        )

    def tearDown(self):
        for f in os.listdir(self.test_dir):
            os.remove(os.path.join(self.test_dir, f))
        os.rmdir(self.test_dir)

    def test_main_convert_to_csv(self):
        output_file = os.path.join(self.test_dir, 'output.csv')
        with patch('sys.argv', ['fit-tool', self.test_fit_file, '-o', output_file]):
            main()
        self.assertTrue(os.path.exists(output_file))

    def test_main_convert_to_fit(self):
        output_file = os.path.join(self.test_dir, 'output.fit')
        with patch('sys.argv', ['fit-tool', self.test_fit_file, '-o', output_file, '-t', 'fit']):
            main()
        self.assertTrue(os.path.exists(output_file))

    def test_main_with_verbose(self):
        output_file = os.path.join(self.test_dir, 'output.csv')
        with patch('sys.argv', ['fit-tool', self.test_fit_file, '-v', '-o', output_file]):
            main()
        self.assertTrue(os.path.exists(output_file))

    def test_main_with_log_file(self):
        output_file = os.path.join(self.test_dir, 'output.csv')
        log_file = os.path.join(self.test_dir, 'test.log')
        with patch('sys.argv', ['fit-tool', self.test_fit_file, '-o', output_file, '-l', log_file]):
            main()
        self.assertTrue(os.path.exists(output_file))
        self.assertTrue(os.path.exists(log_file))

    def test_main_default_output_filename(self):
        original_cwd = os.getcwd()
        try:
            os.chdir(self.test_dir)
            with patch('sys.argv', ['fit-tool', self.test_fit_file]):
                main()
            self.assertTrue(os.path.exists('Activity.csv'))
        finally:
            os.chdir(original_cwd)

    def test_main_infer_format_from_output(self):
        output_file = os.path.join(self.test_dir, 'output.fit')
        with patch('sys.argv', ['fit-tool', self.test_fit_file, '-o', output_file]):
            main()
        self.assertTrue(os.path.exists(output_file))


if __name__ == '__main__':
    unittest.main()



import datetime

import gpxpy
import math
from geopy.distance import geodesic

from fit_tool.fit_file_builder import FitFileBuilder
from fit_tool.profile.messages.event_message import EventMessage
from fit_tool.profile.messages.file_id_message import FileIdMessage
from fit_tool.profile.messages.record_message import RecordMessage
from fit_tool.profile.profile_type import FileType, Manufacturer, Event, EventType


def main():
    """This example shows how to encode an activity into the FIT format and write it to a file. For simplicity, this
    example uses the FitFileBuilder to construct the FIT file, however in practice you might want to encode and
    write record messages to the file immediately for robustness and better memory usage. An example of how to do this
    is in the unit tests.
    """
    now_timestamp_millis = round(datetime.datetime(2022, 5, 10, 5, 5, 5).timestamp()) * 1000

    # Set auto_define to true, so that the builder creates the required Definition Messages for us.
    builder = FitFileBuilder(auto_define=True, min_string_size=50)

    # Read position data from a GPX file
    gpx_file = open('../tests/data/old_stage_left_hand_lee.gpx', 'r')
    gpx = gpxpy.parse(gpx_file)

    message = FileIdMessage()
    message.type = FileType.ACTIVITY
    message.manufacturer = Manufacturer.STAGES_CYCLING.value
    message.product = 0
    message.time_created = now_timestamp_millis
    message.serial_number = 0x12345678
    builder.add(message)

    # It is a best practice to include timer start and stop events in all Activity files. A timer start event
    # should occur before the first Record message in the file, and a timer stop event should occur after the
    # last Record message in the file when the activity recording is complete. Timer stop and start events
    # should be used anytime the activity recording has been paused and resumed. Record messages should not be
    # encoded to the file when the timer is paused.
    start_timestamp = now_timestamp_millis
    message = EventMessage()
    message.event = Event.TIMER
    message.event_type = EventType.START
    message.timestamp = start_timestamp
    builder.add(message)

    distance = 0.0
    timestamp = start_timestamp

    records = []

    prev_coordinate = None

    for index, track_point in enumerate(gpx.tracks[0].segments[0].points):
        current_coordinate = (track_point.latitude, track_point.longitude)

        # calculate distance from previous coordinate and accumulate distance
        if prev_coordinate:
            delta = geodesic(prev_coordinate, current_coordinate).meters
        else:
            delta = 0.0
        distance += delta

        message = RecordMessage()
        message.position_lat = track_point.latitude
        message.position_long = track_point.longitude
        message.distance = distance
        message.timestamp = timestamp
        message.power = round(20 * math.sin(2 * math.pi * index / 50) + 200)
        records.append(message)

        timestamp += 10000
        prev_coordinate = current_coordinate

    builder.add_all(records)

    message = EventMessage()
    message.event = Event.TIMER
    message.event_type = EventType.STOP
    message.timestamp = timestamp
    builder.add(message)

    # Finally build the FIT file object and write it to a file
    fit_file = builder.build()

    out_path = '../tests/out/old_stage_activity.fit'
    fit_file.to_file(out_path)
    csv_path = '../tests/out/old_stage_activity.csv'
    fit_file.to_csv(csv_path)


if __name__ == "__main__":
    main()



# nosetests --nocapture  tests/test_record.py

import unittest

from fit_tool.definition_message import DefinitionMessage
from fit_tool.profile.messages.workout_step_message import WorkoutStepMessage
from fit_tool.record import RecordHeader, Record


class TestRecord(unittest.TestCase):

    def shortDescription(self):
        return None

    def test_normal_record_header(self):
        """Test packing and unpacking of normal record header.
        """
        expected_rh = RecordHeader(is_definition=True,
                                   local_id=RecordHeader.MAX_NORMAL_LOCAL_ID)

        bytes1 = expected_rh.to_bytes()

        rh = RecordHeader.from_bytes(bytes1)
        bytes2 = rh.to_bytes()

        self.assertEqual(bytes2, bytes1)
        self.assertEqual(rh, expected_rh)

    def test_compressed_timestamp_record_header(self):
        """Test packing and unpacking of a compressed timestamp header.
        """
        expected_rh = RecordHeader(is_time_compressed=True,
                                   local_id=3,
                                   time_offset_seconds=10)

        bytes1 = expected_rh.to_bytes()

        rh = RecordHeader.from_bytes(bytes1)
        bytes2 = rh.to_bytes()

        self.assertEqual(bytes2, bytes1)

    def test_record_pack_unpack(self):
        """Test packing and unpacking of a record
        """
        local_id = 15
        dm1 = WorkoutStepMessage(local_id=local_id)
        dm1.workout_step_name = 'test'

        record1 = Record.from_message(dm1)

        bytes1 = record1.to_bytes()

        definition_message = DefinitionMessage.from_data_message(dm1)
        record2 = Record.from_bytes(definition_messages={local_id: definition_message}, bytes_buffer=bytes1)
        bytes2 = record2.to_bytes()

        self.assertEqual(bytes2, bytes1)

    def test_record_to_row(self):
        """Test record to_row
        """
        local_id = 15
        dm1 = WorkoutStepMessage(local_id=local_id)
        dm1.workout_step_name = 'test'

        record1 = Record.from_message(dm1)

        print(record1.to_row())
